;********************************** Beacon code ***************************
; Written 20/7/99
; Ben Hitchcock
;**************************************************************************


.include "1200def.inc"

;***** Pin definitions

.equ	Spare1	=6			 ;Spare
.equ	RedLED	=5		 	;Red LED is PD5
.equ	Spare2	=4			 ;Spare
.equ IRLED = 3    ;IR LED is PD3
.equ Spare3 = 2   ;Spare
.equ Spare4 = 1   ;Spare
.equ Spare5 = 0   ;Spare

.equ Spare6 = 0   ;Spare


;***** Global register variables

.def	BitCnt	=R16			    ;bit counter
.def	StatusByte	=R21			;box status byte

.def	TxByte	=R18		    	;Data to be transmitted
.def	RxByte	=R19	    		;Received data
.def SBits =R20        ;storage for the status of the strobe lines.


;***** Global Masks

.equ GrnButMask = 0x40  ; Mask for Bit 6
.equ SrvStrbMask = 0x20 ; Mask for Bit 5
.equ SrvDataMask = 0x10 ; Mask for Bit 4
.equ BoxDataMask = 0x04 ; Mask for Bit 2
.equ RedButMask  = 0x01 ; Mask for Bit 0

.equ ButtonStateMask = 0x0F ; Mask for the button state in the status byte.


.cseg
.org 0
;*INTERRUPT JUMP TABLE

		rjmp   reset  ;Reset
 	reti          ;IRQ handle
		reti          ;Timer0
		reti	      ;Analog Comp

;***** Program Execution Starts Here

;***** Test program


reset:		
; Set the data direction for the input/output pins.

  cbi	DDRD,Spare1	   ;Set spare as input
  cbi	DDRD,Spare2	   ;Set spare as input
  cbi	DDRD,Spare3	   ;Set spare as input
  cbi	DDRD,Spare4	   ;Set spare as input
  cbi	DDRD,Spare5	   ;Set spare as input
  sbi	DDRD,RedLED	   ;Set Red LED as output
  sbi DDRD, IRLED    ;Set IR LED as output

  cbi DDRB,0    ;Set spare as input
  cbi DDRB,1   ;Set spare as input
  cbi DDRB,2   ;Set spare as input
  cbi DDRB,3   ;Set spare as input
  cbi DDRB,4   ;Set spare as input
  cbi	DDRB,5	   ;Set spare as input
  cbi DDRB,6    ;Set spare as input
  cbi DDRB,7    ;Set spare as input


; Turn off LEDs, turn on pull-ups for buttons, turn off pull-ups for data lines

  sbi PORTD,RedLED   ;Turn the Red LED off
  sbi PORTD,IRLED    ;Turn the IR LED off
  cbi PORTD,Spare1   ;Turn off Spare pull-up
  cbi PORTD,Spare2   ;Turn off Spare pull-up
  cbi PORTD,Spare3   ;Turn off Spare pull-up
  cbi PORTD,Spare4   ;Turn off Spare pull-up
  cbi PORTD,Spare5   ;Turn off Spare pull-up


.equ     OLC =   100               ; outer loop count for delay routine
.equ     ILC =   250             ; inner loop count for delay routine
.equ     IRLC =   123            ; Infra Red Loop Counter (210 cycles)
.equ     BeaconILC =   200             ; inner loop count for beacon
.equ     BeaconOLC =   1            ; outer Loop Counter for beacon

        ldi     r16, $01        ; initially enable just the lowest bit
LOOP:
         rcall  SetRedLED        ;Set the output LED
         rcall  Delay            ;Wait for a bit
         rcall  ClearRedLED      ;Clear the Red LED
         rcall SetBeacon         ;Set the beacon on
         ldi     r22, 50         ; Space between IR flashes
         rcall DL1               ; Delay for a bit
         rcall SetBeacon         ;Set the beacon again
         rcall Delay             ; Delay for a bit
         sbic  ACSR, ACO         ; Low battery routine
         rcall SetRedLED         ; Set the red LED
         rcall Delay             ; Delay for a bit
         rcall ClearRedLED       ; Clear the Red LED
         rcall Delay             ; Delay for a bit
         rcall Delay             ; Delay for a bit
         rcall Delay             ; Delay for a bit
         rcall Delay             ; Delay for a bit
         rcall Delay             ; Delay for a bit
         rcall Delay             ; Delay for a bit
         rcall Delay             ; Delay for a bit
         rcall Delay             ; Delay for a bit
         rjmp  LOOP              ; Start again
         
SetRedLED:
         CBI   PortD, RedLED  ;Set the red LED
         ret;
         
ClearRedLED:
         SBI   PortD, RedLED  ;Clear the red LED
         ret;

SetIRLED:
         SBI   PortD, IRLED  ;Set the IR LED
         ret;
         
ClearIRLED:
         CBI   PortD, IRLED  ;Clear the IR LED
         ret;

SetBeacon:
         ldi   r25, beaconOLC ;BeaconOuterLoopCounter
BeaconDL1:    ldi     r26, BeaconILC
BeaconDL2:    rcall   SetIRLED           ; Set the IR LED
         rcall    IRDelay
         rcall    ClearIRLED           ; Clear the IR LED
         rcall    IRDelay
         dec     r26
         brne    BeaconDL2                             
         dec     R25
         brne    beaconDL1
         ret                           

         
        
StartDelay:        

        ldi     r17, 5          ; wait 0.5 sec
LP1:    rcall   DELAY
        dec     r17
        brne    LP1

        rol     r16             ; rotate the bit
        cpi     r16, $00        ; if all bits not set
        brne    LOOP            ;   ... continue
        inc     r16             ; else set just the lowest bit
        rjmp    LOOP            ; loop forever

DELAY:  ldi     r22, OLC        ; delay for 1/10th second
DL1:    ldi     r23, ILC
DL2:    rcall   DUMMY           ; waste 7 cycles
        dec     r23
        brne    DL2                             
        dec     R22
        brne    DL1
DUMMY:  ret                           

IRDELAY:
         ldi   r24, IRLC      ; Delay for 210 cycles (38 kHz)     1
IRDL1:   dec   r24            ; Do (r24 -= 1) ...                 1
         brne  IRDL1          ; ...while r24 > 0                  1/2
         ret                  ; Return to calling line.           4