#define CheckForResError() if (ResError()) return false

// CPreferencesFile class usage:
//
// Create a CPreferencesFile object, the only constructor takes the name of the preferences file and the creator code
// that you want to use.  A preferences file will be created on disk if it does not already exist.  If it does already
// exist, then it will be opened.
//
// WritePrefs takes a pointer to the data you want to store in the prefs and the size of the data you are passing.
// Any subsequent call to WritePrefs will overwrite any previous data.
//
// ReadPrefs takes a pointer to a pointer.  Pass it the address of the pointer which you want to point to the data
// obtained from the preferences file.  ReadPrefs allocates enough memory for your pointer, assuming it is the
// right type of pointer.
//
// When your CPreferencesFile object goes out of scope, the prefs file will be closed.

class CPreferencesFile
{
	public:
		CPreferencesFile(unsigned char*,OSType,bool*);
		bool WritePrefs(void*,long);
		void ReadPrefs(void**);
		~CPreferencesFile();
	
	private:
		unsigned char* m_fileName;
		void* m_prefsBlock;
		long m_prefsSize;
		int m_prefsRefNum;
		bool m_killOnDestruct;
		
};
