#include "EventSupport.h"
#include "MacAVRpa.h"
#include "MenuSupport.h"
#include "FileSupport.h"
#include "TextWindowSupport.h"

//  global variables

AEEventHandlerUPP	doOpenAppEventUPP;
AEEventHandlerUPP	doReopenAppEventUPP;
AEEventHandlerUPP	doOpenDocsEventUPP;
AEEventHandlerUPP	doPrintDocsEventUPP;
AEEventHandlerUPP	doQuitAppEventUPP;

AEEventHandlerUPP	doAppearanceChangeUPP;
AEEventHandlerUPP	doSysFontChangeUPP;
AEEventHandlerUPP	doSmallSysFontChangeUPP;
AEEventHandlerUPP	doViewsFontChangeUPP;



void doSetupEventUPPs(void)
{

	doOpenAppEventUPP	= NewAEEventHandlerProc((ProcPtr) doOpenAppEvent);
	doReopenAppEventUPP	= NewAEEventHandlerProc((ProcPtr) doReopenAppEvent);
	doOpenDocsEventUPP	= NewAEEventHandlerProc((ProcPtr) doOpenDocsEvent);
	doPrintDocsEventUPP	= NewAEEventHandlerProc((ProcPtr) doPrintDocsEvent);
	doQuitAppEventUPP	= NewAEEventHandlerProc((ProcPtr) doQuitAppEvent);
	
	//  create routine descriptor (Appearance Manager Apple event handlers)

	doAppearanceChangeUPP		= NewAEEventHandlerProc((ProcPtr) doAppearanceChangeEvent);
	
	if(gMacOS85Present)
	{
		doSysFontChangeUPP			= NewAEEventHandlerProc((ProcPtr) doSysFontChangeEvent);
		doSmallSysFontChangeUPP	= NewAEEventHandlerProc((ProcPtr) doSmallSysFontChangeEvent);
		doViewsFontChangeUPP		= NewAEEventHandlerProc((ProcPtr) doViewsFontChangeEvent);
	}

}

//  doInstallAEHandlers

void  doInstallAEHandlers(void)
{
	OSErr	osErr;

	//  required Apple events

	osErr = AEInstallEventHandler(kCoreEventClass,kAEOpenApplication,doOpenAppEventUPP,
																0L,false);
	if(osErr != noErr)	doErrorAlert(eInstallHandler);

	osErr = AEInstallEventHandler(kCoreEventClass,kAEReopenApplication,doReopenAppEventUPP,
																0L,false);
	if(osErr != noErr)	doErrorAlert(eInstallHandler);

	osErr = AEInstallEventHandler(kCoreEventClass,kAEOpenDocuments,doOpenDocsEventUPP,
																0L,false);
	if(osErr != noErr)	doErrorAlert(eInstallHandler);

	osErr = AEInstallEventHandler(kCoreEventClass,kAEPrintDocuments,doPrintDocsEventUPP,
																0L,false);
	if(osErr != noErr)	doErrorAlert(eInstallHandler);

	osErr = AEInstallEventHandler(kCoreEventClass,kAEQuitApplication,doQuitAppEventUPP,
																0L,false);
	if(osErr != noErr)	doErrorAlert(eInstallHandler);

	//  Appearance Manager Apple events

	osErr = AEInstallEventHandler(kAppearanceEventClass,kAEAppearanceChanged,
																doAppearanceChangeUPP,0L,false);
	if(osErr != noErr)	doErrorAlert(eInstallHandler);

	if(gMacOS85Present)
	{
		osErr = AEInstallEventHandler(kAppearanceEventClass,kAESystemFontChanged,
																	doSysFontChangeUPP,0L,false);
		if(osErr != noErr)	doErrorAlert(eInstallHandler);

		osErr = AEInstallEventHandler(kAppearanceEventClass,kAESmallSystemFontChanged,
																	doSmallSysFontChangeUPP,0L,false);
		if(osErr != noErr)	doErrorAlert(eInstallHandler);

		osErr = AEInstallEventHandler(kAppearanceEventClass,kAEViewsFontChanged,
																	doViewsFontChangeUPP,0L,false);
		if(osErr != noErr)	doErrorAlert(eInstallHandler);
	}
}

//  doEvents

void	doEvents(EventRecord *eventStrucPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;
	SInt8			charCode;
	Rect 		growRect;
	SInt32			newSize;

	switch(eventStrucPtr->what)
	{
// For 68k use we don't need these

//		case kHighLevelEvent:
//			AEProcessAppleEvent(eventStrucPtr);
//			break;

		case mouseDown:
			partCode = FindWindow(eventStrucPtr->where,&windowPtr);
			switch(partCode)
			{
				case inMenuBar:
					doAdjustMenus();
					doMenuChoice(MenuSelect(eventStrucPtr->where));
					break;

				case inContent:
					if(windowPtr != FrontWindow())
						SelectWindow(windowPtr);
					else
						doInContent(eventStrucPtr);
					break;

				case inDrag:
					DragWindow(windowPtr,eventStrucPtr->where,&qd.screenBits.bounds);
					doAdjustCursor(windowPtr);
					break;

				case inGoAway:
					if(TrackGoAway(windowPtr,eventStrucPtr->where))
						doCloseWindow(FrontWindow());
					break;


				case inGrow:
					growRect = qd.screenBits.bounds;
					growRect.top 	= 80; 
					growRect.left = 160;

					newSize = GrowWindow(windowPtr,eventStrucPtr->where,&growRect);
					if(newSize != 0)
					{
						SizeWindow(windowPtr,LoWord(newSize),HiWord(newSize),true);
						InvalRect(&windowPtr->portRect);
						doUpdateDocument(windowPtr);	
						doActivateDocWindow(windowPtr,true);			
					}
					break;
			}
			break;

		case keyDown:
		case autoKey:
			charCode = eventStrucPtr->message & charCodeMask;
			if((eventStrucPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuEvent(eventStrucPtr));
			}
			else
				doKeyEvent(charCode);
			break;

		case updateEvt:
			doUpdate(eventStrucPtr);
			break;

		case activateEvt:
			doActivate(eventStrucPtr);
			break;

		case osEvt:
			doOSEvent(eventStrucPtr);
			break;
	}
}

//  doOSEvent

void	doOSEvent(EventRecord *eventStrucPtr)
{
	switch((eventStrucPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			gInBackground = (eventStrucPtr->message & resumeFlag) == 0;
			if(!gInBackground)
				SetCursor(&qd.arrow);
			if(gNumberOfWindows > 0)
			{
				if(gDialogPtr == NULL)
				{ 
					doActivateDocWindow(FrontWindow(),!gInBackground);
				}
			}
			
			HiliteMenu(0);
			break;

		case mouseMovedMessage:
			doAdjustCursor(FrontWindow());
			break;
	}
}


//  doOpenAppEvent

pascal OSErr  doOpenAppEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefCon)
{
	OSErr			osErr;

//	gApplicationWasOpen = true;

	osErr = doHasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{
		doNewDocWindow();
		return(noErr);
	}
	else
		return(osErr);
}

//  doReopenAppEvent

pascal OSErr  doReopenAppEvent(AppleEvent *appEvent,AppleEvent *reply,
															 SInt32 handlerRefCon)
{
	OSErr			osErr;

	osErr = doHasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{

		return(noErr);
	}
	else
		return(osErr);
}

//  doOpenDocsEvent

pascal OSErr  doOpenDocsEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefcon)
{
	AEDescList	docList;
	OSErr				osErr, ignoreErr;
	SInt32			numberOfItems, index;
	DescType		returnedType;
	FSSpec			fileSpec;
	AEKeyword		keyWord;
	Size				actualSize;

	osErr = AEGetParamDesc(appEvent,keyDirectObject,typeAEList,&docList);

	if(osErr == noErr)
	{
		osErr = doHasGotRequiredParams(appEvent);
		if(osErr == noErr)
		{
			AECountItems(&docList,&numberOfItems);
			if(osErr == noErr)
			{
				for(index=1;index<=numberOfItems;index++)
				{
					osErr = AEGetNthPtr(&docList,index,typeFSS,&keyWord,&returnedType,
															(Ptr) &fileSpec,sizeof(fileSpec),&actualSize);
					if(osErr == noErr)
					{
						doOpenFile(fileSpec);
					}
					else
						doErrorAlert(eGetDescriptorRecord);
				}
			}
		}
		else
			doErrorAlert(eMissedRequiredParam);

		ignoreErr = AEDisposeDesc(&docList);
	}
	else
		doErrorAlert(eGetRequiredParam);

	return(osErr);
}

//  doPrintDocsEvent

pascal OSErr  doPrintDocsEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefcon)
{
	AEDescList	docList;
	OSErr				osErr, ignoreErr;
	SInt32			numberOfItems, index;
	DescType		returnedType;
	
	osErr = AEGetParamDesc(appEvent,keyDirectObject,typeAEList,&docList);

/*	if(osErr == noErr)
	{
		osErr = doHasGotRequiredParams(appEvent);
		if(osErr == noErr)
		{
			AECountItems(&docList,&numberOfItems);
			if(osErr == noErr)
			{
				for(index=1;index<=numberOfItems;index++)
				{
					osErr = AEGetNthPtr(&docList,index,typeFSS,&keyWord,&returnedType,
															(Ptr) &fileSpec,sizeof(fileSpec),&actualSize);
					if(osErr == noErr)
					{
						if(!(result = doPrintFile(&fileSpec,index,numberOfItems)))
							doErrorAlert(eCannotPrintFile);
					}
					else
						doErrorAlert(eGetDescriptorRecord);
				}
			}
		}
		else
			doErrorAlert(eMissedRequiredParam);

		ignoreErr = AEDisposeDesc(&docList);
	}
	else
		doErrorAlert(eGetRequiredParam);
*/
	return(osErr);
}

//  doQuitAppEvent

pascal OSErr  doQuitAppEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefcon)
{
	OSErr		osErr;

	osErr = doHasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{
		gDone = true;
		return(noErr);
	}
	else
		return(osErr);
}

//  doAppearanceChangeEvent

pascal OSErr  doAppearanceChangeEvent(AppleEvent *appEvent,AppleEvent *reply,
																			SInt32 handlerRefcon)
{
	OSErr	osErr;

	osErr = doHasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{
		// Action as required by application.
		return(osErr);
	}
	else
		return(osErr);
}

//  doSysFontChangeEvent

pascal OSErr  doSysFontChangeEvent(AppleEvent *appEvent,AppleEvent *reply,
																	 SInt32 handlerRefcon)
{
	OSErr		osErr;
	Str255	fontName, theString = "\p   Current large system font is: ";

	osErr = doHasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{
//		doDrawText("\pReceived an Apple event: LARGE SYSTEM FONT CHANGED.");
#if TARGET_CPU_PPC
//		GetThemeFont(kThemeSystemFont,smSystemScript,fontName,NULL,NULL);
//		doConcatPStrings(theString,fontName);
//		doDrawText(theString);
#endif
		// Action as required by application.
		return(osErr);
	}
	else
		return(osErr);
}

//  doSmallSysFontChangeEvent

pascal OSErr  doSmallSysFontChangeEvent(AppleEvent *appEvent,AppleEvent *reply,
																				SInt32 handlerRefcon)
{
	OSErr	osErr;
	Str255	fontName, theString = "\p   Current small system font is: ";
	
	osErr = doHasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{
//		doDrawText("\pReceived an Apple event: SMALL SYSTEM FONT CHANGED.");
#if TARGET_CPU_PPC
//		GetThemeFont(kThemeSmallSystemFont,smSystemScript,fontName,NULL,NULL);
//		doConcatPStrings(theString,fontName);
//		doDrawText(theString);
#endif
		// Action as required by application.
	}
	else
		return(osErr);
}

//  doViewsFontChangeEvent

pascal OSErr  doViewsFontChangeEvent(AppleEvent *appEvent,AppleEvent *reply,
																			SInt32 handlerRefcon)
{
	OSErr		osErr;
	Str255	fontName, fontSizeString, theString = "\p   Current views font is: ";
	SInt16	fontSize;

	osErr = doHasGotRequiredParams(appEvent);

	if(osErr == noErr)
	{
//		doDrawText("\pReceived an Apple event: VIEWS FONT CHANGED.");
#if TARGET_CPU_PPC
//		GetThemeFont(kThemeViewsFont,smSystemScript,fontName,&fontSize,NULL);
//		doConcatPStrings(theString,fontName);
//		doConcatPStrings(theString,"\p ");
//		NumToString((SInt32) fontSize,fontSizeString);
//		doConcatPStrings(theString,fontSizeString);
//		doConcatPStrings(theString,"\p point");
//		doDrawText(theString);
#endif
		// Action as required by application.
		return(osErr);
	}
	else
		return(osErr);
}

//  doHasGotRequiredParams

OSErr  doHasGotRequiredParams(AppleEvent *appEvent)
{
	OSErr			osErr;	
	DescType	returnedType;
	Size			actualSize;

	osErr = AEGetAttributePtr(appEvent,keyMissedKeywordAttr,typeWildCard,&returnedType,
														NULL,0,&actualSize);

	if(osErr == errAEDescNotFound)
		return(noErr);
	else if(osErr == noErr)
		return(errAEParamMissed);
}

