// 
// HelpDialog.c
// 

#include <Appearance.h>
#include <ControlDefinitions.h>
#include <Dialogs.h>
#include <Resources.h>
#include <TextUtils.h>

//  defines

#define rHelpModal						128
#define  iUserPane						2
#define  iScrollBar						3
#define  iPopupMenu						4
#define  eHelpDialog					9
#define  eHelpDocRecord				10
#define  eHelpText						11
#define  eHelpPicture					12
#define rTextIntroduction			128
#define rTextCreatingText			129
#define rTextModifyHelp				130
#define rPictIntroductionBase	128
#define rPictCreatingTextBase	129
#define kTextInset						4

//  typedefs

typedef struct
{
	Rect			bounds;
	PicHandle	pictureHdl;
} pictInfoStructure;

typedef struct
{
	TEHandle					editRecHdl;
	ControlHandle			scrollbarHdl;
	SInt16						pictCount;
	pictInfoStructure	*pictInfoStructurePtr;
}	docStructure, ** docStructureHandle;

typedef struct
{
	RGBColor			backColour;
	PixPatHandle	backPixelPattern;
	Pattern				backBitPattern;
} backColourPattern;

//  global variables

ModalFilterUPP					eventFilterUPP;
ControlUserPaneDrawUPP	userPaneDrawFunctionUPP;
ControlActionUPP				actionFunctionUPP;
backColourPattern 			gBackColourPattern;
SInt16									gTextResourceID;
SInt16									gPictResourceBaseID;
RgnHandle								gSavedClipRgn	= NULL;

extern Boolean					gMacOS85Present;

//  function prototypes

void						doHelp								(void);
void						doCloseHelp						(DialogPtr,GrafPtr);
void						doDisposeDescriptors	(void);
pascal void  		userPaneDrawFunction	(ControlHandle,SInt16);
Boolean					doGetText							(DialogPtr,SInt16,Rect);
Boolean					doGetPictureInfo			(DialogPtr,SInt16);
pascal void			actionFunction				(ControlHandle,SInt16);
void						doScrollTextAndPicts	(DialogPtr);
void 						doDrawPictures				(DialogPtr,Rect *);
pascal Boolean	eventFilter						(DialogPtr,EventRecord *,SInt16 *);
void						doSaveBackground			(backColourPattern *);
void						doRestoreBackground		(backColourPattern *);
void						doSetBackgroundWhite	(void);

extern void			doUpdate							(EventRecord *);
extern void			doErrorAlert					(SInt16);

//  doHelp

void  doHelp(void)
{
	DialogPtr						dialogPtr;
	docStructureHandle	docStrucHdl;
	GrafPtr							oldPort;
	ControlHandle				controlHdl;
	SInt16							itemHit, menuItem;
	Rect								userItemRect, destRect, viewRect;

	//  create routine descriptors

	eventFilterUPP					= NewModalFilterProc((ProcPtr) eventFilter);
	userPaneDrawFunctionUPP	= NewControlUserPaneDrawProc((ProcPtr) userPaneDrawFunction);
	actionFunctionUPP				= NewControlActionProc((ProcPtr) actionFunction);

	//  create dialog and attach document structure

	if(!(dialogPtr = GetNewDialog(rHelpModal,NULL,(WindowPtr) -1)))
	{
		doErrorAlert(eHelpDialog);
		doDisposeDescriptors();
		return;
	}

	if(!(docStrucHdl = (docStructureHandle) NewHandle(sizeof(docStructure))))
	{
		doErrorAlert(eHelpDocRecord);
		DisposeDialog(dialogPtr);
		doDisposeDescriptors();
		return;
	}

	SetWRefCon(dialogPtr,(SInt32) docStrucHdl);

	//  set graphics port and default button
	
	GetPort(&oldPort);
	SetPort(dialogPtr);
	SetDialogDefaultItem(dialogPtr,kStdOkItemIndex);

	//  set user pane drawing function
	
	GetDialogItemAsControl(dialogPtr,iUserPane,&controlHdl);
	SetControlData(controlHdl,kControlNoPart,kControlUserPaneDrawProcTag, 
								 sizeof(userPaneDrawFunctionUPP),(Ptr) &userPaneDrawFunctionUPP);

	//  set destination and view rectangles, create edit structure

	userItemRect = (*controlHdl)->contrlRect;
	InsetRect(&userItemRect,kTextInset,kTextInset / 2);
	destRect = viewRect = userItemRect;
	(*docStrucHdl)->editRecHdl = TEStyleNew(&destRect,&viewRect);

	//  assign handle to scroll bar to relevant document structure field 

	GetDialogItemAsControl(dialogPtr,iScrollBar,&controlHdl);
	(*docStrucHdl)->scrollbarHdl = controlHdl;

	//  initialise picture information structure field of document structure 

	(*docStrucHdl)->pictInfoStructurePtr = NULL;

	//  assign resource IDs of first topic's 'TEXT'/'styl' resources
	
	gTextResourceID			= rTextIntroduction;
	gPictResourceBaseID	= rPictIntroductionBase;

	//  load text resources and insert into edit structure

	if(!(doGetText(dialogPtr,gTextResourceID,viewRect)))
	{
		doCloseHelp(dialogPtr,oldPort);
		doDisposeDescriptors();
		return;
	}

	//  search for option-space charas in text and load same number of 'PICT' resources

	if(!(doGetPictureInfo(dialogPtr,gPictResourceBaseID)))
	{
		doCloseHelp(dialogPtr,oldPort);
		doDisposeDescriptors();
		return;
	}

	//  create an empty region for saving the old clipping region
		
	gSavedClipRgn = NewRgn();

	//  show window and save background colour and pattern

	ShowWindow(dialogPtr);

	doSaveBackground(&gBackColourPattern);

	//  enter ModalDialog loop

	do 
	{
		ModalDialog(eventFilterUPP,&itemHit);
	
		if(itemHit == iPopupMenu)
		{
			SetControlValue((*docStrucHdl)->scrollbarHdl,0);

			GetDialogItemAsControl(dialogPtr,iPopupMenu,&controlHdl);
			menuItem = GetControlValue(controlHdl);
			
			switch(menuItem)
			{
				case 1:
					gTextResourceID 		= rTextIntroduction;
					gPictResourceBaseID = rPictIntroductionBase;
					break;
				
				case 2:
					gTextResourceID 		= rTextCreatingText;
					gPictResourceBaseID = rPictCreatingTextBase;
					break;

				case 3:
					gTextResourceID 		= rTextModifyHelp;
					break;
			}
	
			doSetBackgroundWhite();
			
			if(!(doGetText(dialogPtr,gTextResourceID,viewRect)))
			{
				doCloseHelp(dialogPtr,oldPort);
				doDisposeDescriptors();
				return;
			}
			if(!(doGetPictureInfo(dialogPtr,gPictResourceBaseID)))
			{
				doCloseHelp(dialogPtr,oldPort);
				doDisposeDescriptors();
				return;
			}

			doDrawPictures(dialogPtr,&viewRect);
			
			doRestoreBackground(&gBackColourPattern);
		}
			
	} while(itemHit != kStdOkItemIndex);

	//  clean up
		
	doCloseHelp(dialogPtr,oldPort);
	doDisposeDescriptors();
}

//  doCloseHelp

void  doCloseHelp(DialogPtr dialogPtr,GrafPtr oldPort)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							a;
	
	docStrucHdl = (docStructureHandle) GetWRefCon(dialogPtr);
	editRecHdl = (*docStrucHdl)->editRecHdl;

	if(gSavedClipRgn)
		DisposeRgn(gSavedClipRgn);

	if((*docStrucHdl)->editRecHdl)
		TEDispose((*docStrucHdl)->editRecHdl);

	if((*docStrucHdl)->pictInfoStructurePtr)
	{
		for(a=0;a<(*docStrucHdl)->pictCount;a++)
			ReleaseResource((Handle) (*docStrucHdl)->pictInfoStructurePtr[a].pictureHdl);
		DisposePtr((Ptr) (*docStrucHdl)->pictInfoStructurePtr);
	}

	DisposeHandle((Handle) docStrucHdl);
	DisposeDialog(dialogPtr);

	SetPort(oldPort);
}

//  doDisposeDescriptors

void  doDisposeDescriptors(void)
{
	DisposeRoutineDescriptor(eventFilterUPP);
	DisposeRoutineDescriptor(userPaneDrawFunctionUPP);
	DisposeRoutineDescriptor(actionFunctionUPP);
}

//  userPaneDrawFunction

pascal void  userPaneDrawFunction(ControlHandle controlHdl,SInt16 thePart)
{
	Rect								itemRect, viewRect;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	DialogPtr						dialogPtr;
	Boolean 						inState;

	dialogPtr = (*controlHdl)->contrlOwner;
	itemRect = (*controlHdl)->contrlRect;
	InsetRect(&itemRect,1,1);
	itemRect.right += 15;

	DrawThemeListBoxFrame(&itemRect,kThemeStateActive);
	if(((WindowPeek) dialogPtr)->visible)
		inState = ((WindowPeek) dialogPtr)->hilited;
	DrawThemeListBoxFrame(&itemRect,inState);

	doSetBackgroundWhite();
	EraseRect(&itemRect);
	
	docStrucHdl = (docStructureHandle) GetWRefCon(dialogPtr);
	editRecHdl = (*docStrucHdl)->editRecHdl;
	viewRect = (*editRecHdl)->viewRect;

	TEUpdate(&viewRect,editRecHdl);
	doDrawPictures(dialogPtr,&viewRect);

	doRestoreBackground(&gBackColourPattern);	
}

//  doGetText

Boolean  doGetText(DialogPtr dialogPtr,SInt16 textResourceID,Rect viewRect)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	Handle							helpTextHdl;
	StScrpHandle				stylScrpRecHdl;
	SInt16							numberOfLines, heightOfText, heightToScroll;

	docStrucHdl = (docStructureHandle) GetWRefCon(dialogPtr);
	editRecHdl = (*docStrucHdl)->editRecHdl;

	TESetSelect(0,32767,editRecHdl);
	TEDelete(editRecHdl);
	
	(*editRecHdl)->destRect = (*editRecHdl)->viewRect;
	SetControlValue((*docStrucHdl)->scrollbarHdl,0);
			
	helpTextHdl = GetResource('TEXT',textResourceID);
	if(helpTextHdl == NULL)
	{
		doErrorAlert(eHelpText);
		return false;
	}

	stylScrpRecHdl = (StScrpHandle) GetResource('styl',textResourceID);
	if(stylScrpRecHdl == NULL)
	{
		doErrorAlert(eHelpText);
		return false;
	}
		
	TEStyleInsert(*helpTextHdl,GetHandleSize(helpTextHdl),stylScrpRecHdl,editRecHdl);

	ReleaseResource(helpTextHdl);
	ReleaseResource((Handle) stylScrpRecHdl);

	numberOfLines = (*editRecHdl)->nLines;
	heightOfText = TEGetHeight((SInt32) numberOfLines,1,editRecHdl);

	if(heightOfText > (viewRect.bottom - viewRect.top))
	{
		heightToScroll = TEGetHeight((SInt32) numberOfLines,1,editRecHdl) -
																 (viewRect.bottom - viewRect.top);
		SetControlMaximum((*docStrucHdl)->scrollbarHdl,heightToScroll);
		ActivateControl((*docStrucHdl)->scrollbarHdl);
#if TARGET_CPU_PPC && !defined MACOS_800
		if(gMacOS85Present)
		{
			SetControlViewSize((*docStrucHdl)->scrollbarHdl,(*editRecHdl)->viewRect.bottom - 
											 	(*editRecHdl)->viewRect.top);
		}
#endif
	}
	else
	{
		DeactivateControl((*docStrucHdl)->scrollbarHdl);
	}

	return true;
}

//  doGetPictureInfo

Boolean  doGetPictureInfo(DialogPtr dialogPtr,SInt16 firstPictID)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	Handle							textHdl;
	SInt32							offset, textSize;
	SInt16							numberOfPicts, a, lineHeight, fontAscent;
//	SInt8								optionSpace[1] = "\xCA";
	SInt8								optionSpace[5] = "\xCA";
	pictInfoStructure		*pictInfoPtr;
	Point								picturePoint;
	TextStyle						whatStyle;
	
	docStrucHdl = (docStructureHandle) GetWRefCon(dialogPtr);

	if((*docStrucHdl)->pictInfoStructurePtr != NULL)
	{
		for(a=0;a<(*docStrucHdl)->pictCount;a++)
			ReleaseResource((Handle) (*docStrucHdl)->pictInfoStructurePtr[a].pictureHdl);

		DisposePtr((Ptr) (*docStrucHdl)->pictInfoStructurePtr);
		(*docStrucHdl)->pictInfoStructurePtr = NULL;
	}

	(*docStrucHdl)->pictCount = 0;

	editRecHdl = (*docStrucHdl)->editRecHdl;
	textHdl = (*editRecHdl)->hText;

	textSize = GetHandleSize(textHdl);
	offset = 0;
	numberOfPicts = 0;

	HLock(textHdl);

	offset = Munger(textHdl,offset,optionSpace,1,NULL,0);
	while((offset >= 0) && (offset <= textSize))
	{
		numberOfPicts++;
		offset++;
		offset = Munger(textHdl,offset,optionSpace,1,NULL,0);		
	}
	
	if(numberOfPicts == 0)
	{
		HUnlock(textHdl);
		return true;
	}
		
	pictInfoPtr = (pictInfoStructure *) NewPtr(sizeof(pictInfoStructure) * numberOfPicts);
	(*docStrucHdl)->pictInfoStructurePtr = pictInfoPtr;

	offset = 0L;

	for(a=0;a<numberOfPicts;a++)
	{
		pictInfoPtr[a].pictureHdl = GetPicture(firstPictID + a);
		if(pictInfoPtr[a].pictureHdl == NULL)
		{
			doErrorAlert(eHelpPicture);
			return false;
		}

		offset = Munger(textHdl,offset,optionSpace,1,NULL,0);
		picturePoint = TEGetPoint((SInt16)offset,editRecHdl);

		TEGetStyle(offset,&whatStyle,&lineHeight,&fontAscent,editRecHdl);
		picturePoint.v -= lineHeight;
		offset++;
		pictInfoPtr[a].bounds = (**pictInfoPtr[a].pictureHdl).picFrame;

		OffsetRect(&pictInfoPtr[a].bounds,
							 (((*editRecHdl)->destRect.right + (*editRecHdl)->destRect.left) -
							 (pictInfoPtr[a].bounds.right + pictInfoPtr[a].bounds.left) ) / 2,
							 - pictInfoPtr[a].bounds.top + picturePoint.v);
	}
	
	(*docStrucHdl)->pictCount = a;
	
	HUnlock(textHdl);

	return true;
}

//  actionFunction

pascal void  actionFunction(ControlHandle scrollbarHdl,SInt16 partCode)
{
	docStructureHandle	docStrucHdl;
	DialogPtr						dialogPtr;
	TEHandle						editRecHdl;
	SInt16							delta, oldValue, offset, lineHeight, fontAscent;
	Point								thePoint;
	Rect								viewRect;
	TextStyle						style;

	if(partCode)
	{
		dialogPtr = (*scrollbarHdl)->contrlOwner;
		docStrucHdl = (docStructureHandle) GetWRefCon(dialogPtr);
		editRecHdl = (*docStrucHdl)->editRecHdl;
		viewRect = (*editRecHdl)->viewRect;
		thePoint.h = viewRect.left + kTextInset;
		
		if(partCode != kControlIndicatorPart)
		{		
			switch(partCode)
			{
				case kControlUpButtonPart:
					thePoint.v = viewRect.top - 4;
					offset = TEGetOffset(thePoint,editRecHdl);
					thePoint = TEGetPoint(offset,editRecHdl);
					TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
					delta = thePoint.v - lineHeight - viewRect.top;
					break;

				case kControlDownButtonPart:
					thePoint.v = viewRect.bottom + 2;
					offset = TEGetOffset(thePoint,editRecHdl);
					thePoint = TEGetPoint(offset,editRecHdl);
					delta = thePoint.v - viewRect.bottom;
					break;

				case kControlPageUpPart:
					thePoint.v = viewRect.top + 2;
					offset = TEGetOffset(thePoint,editRecHdl);
					thePoint = TEGetPoint(offset,editRecHdl);
					TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
					thePoint.v += lineHeight - fontAscent;
					thePoint.v -= viewRect.bottom - viewRect.top;
					offset = TEGetOffset(thePoint,editRecHdl);
					thePoint = TEGetPoint(offset,editRecHdl);
					TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
					delta = thePoint.v - viewRect.top;
					if(offset == 0)
						delta -= lineHeight;
					break;
				
				case kControlPageDownPart:
					thePoint.v = viewRect.bottom - 2;
					offset = TEGetOffset(thePoint,editRecHdl);
					thePoint = TEGetPoint(offset,editRecHdl);
					TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
					thePoint.v -= fontAscent;
					thePoint.v += viewRect.bottom - viewRect.top;
					offset = TEGetOffset(thePoint,editRecHdl);
					thePoint = TEGetPoint(offset,editRecHdl);
					TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
					delta =  thePoint.v - lineHeight - viewRect.bottom;
					if(offset == (**editRecHdl).teLength)
						delta += lineHeight;
					break;
			}

			oldValue = GetControlValue(scrollbarHdl);

			if(((delta < 0) && (oldValue > 0)) || ((delta > 0) && 
				 (oldValue < GetControlMaximum(scrollbarHdl))))
			{
 				GetClip(gSavedClipRgn);
				ClipRect(&dialogPtr->portRect);

				SetControlValue(scrollbarHdl,oldValue + delta);
				SetClip(gSavedClipRgn);
			}
		}
		
		doScrollTextAndPicts(dialogPtr);
	}
}

//  doScrollTextAndPicts

void  doScrollTextAndPicts(DialogPtr dialogPtr)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							scrollDistance, oldScroll;
	Rect								updateRect;

	doSetBackgroundWhite();

	docStrucHdl = (docStructureHandle) GetWRefCon(dialogPtr);
	editRecHdl = (*docStrucHdl)->editRecHdl;

	oldScroll = (*editRecHdl)->viewRect.top -(*editRecHdl)->destRect.top;
	scrollDistance = oldScroll - GetControlValue((*docStrucHdl)->scrollbarHdl);
	if(scrollDistance == 0)
	{
		doRestoreBackground(&gBackColourPattern);	
		return;
	
	}
	TEScroll(0,scrollDistance,editRecHdl);

	if((*docStrucHdl)->pictCount == 0)
	{
		doRestoreBackground(&gBackColourPattern);	
		return;
	}
		
	updateRect = (*editRecHdl)->viewRect;

	if(scrollDistance > 0)
	{
		if(scrollDistance < (updateRect.bottom - updateRect.top))
			updateRect.bottom = updateRect.top + scrollDistance;
	}
	else
	{
		if( - scrollDistance < (updateRect.bottom - updateRect.top))
			updateRect.top = updateRect.bottom + scrollDistance;
	}

	doDrawPictures(dialogPtr,&updateRect);

	doRestoreBackground(&gBackColourPattern);	
}

//  doDrawPictures

void  doDrawPictures(DialogPtr dialogPtr,Rect *updateRect)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							pictCount, pictIndex, vOffset;
	PicHandle						thePictHdl;
	Rect								pictLocRect, dummyRect;

	docStrucHdl = (docStructureHandle) GetWRefCon(dialogPtr);
	editRecHdl = (*docStrucHdl)->editRecHdl;

	vOffset = (*editRecHdl)->destRect.top - (*editRecHdl)->viewRect.top - kTextInset;
	pictCount = (*docStrucHdl)->pictCount;

	for(pictIndex = 0;pictIndex < pictCount;pictIndex++)
	{
		pictLocRect = (*docStrucHdl)->pictInfoStructurePtr[pictIndex].bounds;
    OffsetRect(&pictLocRect,0,vOffset);

		if(!SectRect(&pictLocRect,updateRect,&dummyRect))
			continue;

		thePictHdl = (*docStrucHdl)->pictInfoStructurePtr[pictIndex].pictureHdl;

		LoadResource((Handle) thePictHdl);
		HLock((Handle) thePictHdl);

		GetClip(gSavedClipRgn);
		ClipRect(updateRect);
		DrawPicture(thePictHdl,&pictLocRect);

		SetClip(gSavedClipRgn);
		HUnlock((Handle) thePictHdl);
	}
}

//  eventFilter

pascal Boolean  eventFilter(DialogPtr dialogPtr,EventRecord *eventStrucPtr,
														SInt16 *itemHit)
{
	Boolean							handledEvent;
	GrafPtr							oldPort;
	Point								mouseXY;
	ControlHandle				controlHdl;
	docStructureHandle	docStrucHdl;

	handledEvent = false;

	if((eventStrucPtr->what == updateEvt) && 
		 ((WindowPtr) eventStrucPtr->message != dialogPtr))
	{
		doUpdate(eventStrucPtr);
	}
	else
	{
		GetPort(&oldPort);
		SetPort(dialogPtr);

		if(eventStrucPtr->what == mouseDown)
		{
			mouseXY = eventStrucPtr->where;
			GlobalToLocal(&mouseXY);
			if(FindControl(mouseXY,dialogPtr,&controlHdl))
			{
				docStrucHdl = (docStructureHandle) GetWRefCon(dialogPtr);
				if(controlHdl == (*docStrucHdl)->scrollbarHdl)
				{
					TrackControl((*docStrucHdl)->scrollbarHdl,mouseXY,actionFunctionUPP);	
					*itemHit = iScrollBar;
					handledEvent = true;
				}
			}
		}
		else
		{
			handledEvent = StdFilterProc(dialogPtr,eventStrucPtr,itemHit);
		}

		SetPort(oldPort);
	}

	return(handledEvent);
}

//  doSaveBackground

void  doSaveBackground(backColourPattern *gBackColourPattern)
{
	GrafPtr	currentPort;
	
	GetPort(&currentPort);

	GetBackColor(&gBackColourPattern->backColour);
	gBackColourPattern->backPixelPattern	= NULL;

	if((**((CGrafPtr) currentPort)->bkPixPat).patType != 0)
		gBackColourPattern->backPixelPattern = ((CGrafPtr) currentPort)->bkPixPat;
	else
		gBackColourPattern->backBitPattern = 
														 *(PatPtr) (*(**((CGrafPtr) currentPort)->bkPixPat).patData);
}

//  doRestoreBackground

void  doRestoreBackground(backColourPattern *gBackColourPattern)
{
	RGBBackColor(&gBackColourPattern->backColour);

	if(gBackColourPattern->backPixelPattern)
		BackPixPat(gBackColourPattern->backPixelPattern);
	else
		BackPat(&gBackColourPattern->backBitPattern);
}

//  doSetBackgroundWhite

void  doSetBackgroundWhite(void)
{
	RGBColor	whiteColour = { 0xFFFF, 0xFFFF, 0xFFFF };

	RGBBackColor(&whiteColour);
	BackPat(&qd.white);
}

// 


