// 
// MacAVRpa.c
// 
//
// This program is based on:
// avrp - a programmer for linux, dos etc by Jon Anders Haugum
// avra - an assembler for linux, dos etc by Jon Anders Haugum
// Text1 - a bare bones text editor code base from MacTech magazine
//



// 

//  includes
#include "unistd.mac.h"
#include <DriverServices.h>
#include <TextUtils.h>
#include <CrmSerialDevices.h>
#include <CommResources.h>

#include "macAVRpa.h"
#include "EventSupport.h"
#include "Miscellany.h"
#include "MenuSupport.h"
#include "FileSupport.h"
#include "TextWindowSupport.h"
#include "avra.h"
#include "avrp.h"

//  global variables

ModalFilterUPP				dialogEventFilterUPP;
Boolean						gMacOS85Present = false;
Boolean						gDone;
Boolean						gInBackground;
RgnHandle					gCursorRegion;
SInt16						gNumberOfWindows = 0;
SInt16 						gOldControlValue;

AVRPrefs					gThePrefs = {9600,1,1,false,false,1};
// Default prefs here.  Port speed is 9600, serial port is the first available one (usually
// modem), Reverse Signature is off, Ignore Signature is off, Maximum Errors is 1

CPreferencesFile			*gThePreferencesFile=NULL;
Str255						gChipList[kNumChipsSupported] =
{ "\pAT89S8252","\pAT90S1200-C","\pAT90S1200-D,E,F","\pAT90S2313","\pAT90S2323","\pAT90S2333","\pAT90S2343","\pAT90S4414",
"\pAT90S4433","\pAT90S4434","\pAT90S8515","\pAT90S8534","\pAT90S8535"};


//  main

void  main(void)
{
	OSErr				osError;
	SInt32			response;
	
	// initialise managers


	doInitManagers();

	//  check whether Mac OS 8.5 or later is present

	osError = Gestalt(gestaltSystemVersion,&response);
	
	if(osError == noErr && response >= 0x00000850)
		gMacOS85Present = true;

	//  create routine descriptors

	doSetupTextUPPs();
	dialogEventFilterUPP = NewModalFilterProc((ProcPtr) dialogEventFilter);
	
	doSetupEventUPPs();

	//  set up menu bar and menus

	doSetupMenus();
	
	//  install Apple event handlers
	
	doInstallAEHandlers();
	
	
	//  load preferences
	CheckPreferences();
	
	unsigned char* blah=&gChipList[0][0];
	
//	doNewDocWindow();
	//  enter eventLoop	
	
	eventLoop();
	


	gThePreferencesFile->WritePrefs(&gThePrefs,sizeof(AVRPrefs));
	delete gThePreferencesFile;
	
	ExitToShell();

}

	
	
//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();

	FlushEvents(everyEvent,0);
	
//	RegisterAppearanceClient();    Commented out for 68K compliance
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord	eventStructure;
	Boolean			gotEvent;
	SInt32			sleepTime;

	gDone = false;
	gCursorRegion = NewRgn();
	sleepTime = LMGetCaretTime();

	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventStructure,sleepTime,gCursorRegion);

		if(gotEvent)
			doEvents(&eventStructure);
		else
		{
			if(gNumberOfWindows > 0)
				doIdle();
		}
	}
}

//  doIdle

void  doIdle(void)
{
	docStructureHandle	docStrucHdl;
	WindowPtr			windowPtr;
	
	windowPtr = FrontWindow();

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	if(docStrucHdl != NULL)
		TEIdle((*docStrucHdl)->editRecHdl);
}	

//  doUpdate

void  doUpdate(EventRecord *eventStrucPtr)
{
	WindowPtr	windowPtr;

	windowPtr = (WindowPtr) eventStrucPtr->message;

	if(!(IsDialog(windowPtr)))
		doUpdateDocument(windowPtr);
	else
		doUpdateModelessDialog(eventStrucPtr);
}


//  doUpdateModelessDialog

void  doUpdateModelessDialog(EventRecord *eventStrucPtr)
{
	WindowPtr windowPtr;
	SInt16		itemHit;

	DialogSelect(eventStrucPtr,&windowPtr,&itemHit);
}

//  doActivate

void  doActivate(EventRecord *eventStrucPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventStrucPtr->message;
	becomingActive = ((eventStrucPtr->modifiers & activeFlag) == activeFlag);

	if(!IsDialog(windowPtr))
		doActivateDocWindow(windowPtr,becomingActive);
	else
		doActivateModelessDialog(eventStrucPtr,becomingActive);

}


//  doActivateModelessDialog

void  doActivateModelessDialog(EventRecord *eventStrucPtr,Boolean becomingActive)
{
	WindowPtr windowPtr;
	SInt16 itemHit;	

	DialogSelect(eventStrucPtr,&windowPtr,&itemHit);

	if(becomingActive)
	{
		doAdjustMenus();
	}
}


//  doErrorAlert

void  doErrorAlert(SInt16 errorCode)
{
	AlertStdAlertParamRec	paramRec;
	Str255								errorString;
	SInt16								itemHit;

	paramRec.movable				= true;
	paramRec.helpButton			= false;
	paramRec.filterProc			= NULL;
	paramRec.defaultText		= (StringPtr) kAlertDefaultOKText;
	paramRec.cancelText			= NULL;
	paramRec.otherText			= NULL;
	paramRec.defaultButton	= kAlertStdAlertOKButton;
	paramRec.cancelButton		= 0;
	paramRec.position				= kWindowDefaultPosition;

	GetIndString(errorString,rErrorStrings,errorCode);

	if(errorCode < eWindow)
	{
		StandardAlert(kAlertStopAlert,errorString,NULL,&paramRec,&itemHit);
		ExitToShell();
	}
	else
		StandardAlert(kAlertCautionAlert,errorString,NULL,&paramRec,&itemHit);
}

//  doErrorAlert

void  doCautionAlert(Str255 theAlertString)
{
	AlertStdAlertParamRec	paramRec;
	SInt16								itemHit;

	paramRec.movable				= true;
	paramRec.helpButton			= false;
	paramRec.filterProc			= NULL;
	paramRec.defaultText		= (StringPtr) kAlertDefaultOKText;
	paramRec.cancelText			= NULL;
	paramRec.otherText			= NULL;
	paramRec.defaultButton	= kAlertStdAlertOKButton;
	paramRec.cancelButton		= 0;
	paramRec.position				= kWindowDefaultPosition;

	StandardAlert(kAlertCautionAlert,theAlertString,NULL,&paramRec,&itemHit);
}

//  doErrorAlert

void  doNoteAlert(Str255 theAlertString)
{
	AlertStdAlertParamRec	paramRec;
	SInt16								itemHit;

	paramRec.movable				= true;
	paramRec.helpButton			= false;
	paramRec.filterProc			= NULL;
	paramRec.defaultText		= (StringPtr) kAlertDefaultOKText;
	paramRec.cancelText			= NULL;
	paramRec.otherText			= NULL;
	paramRec.defaultButton	= kAlertStdAlertOKButton;
	paramRec.cancelButton		= 0;
	paramRec.position				= kWindowDefaultPosition;

	StandardAlert(kAlertNoteAlert,theAlertString,NULL,&paramRec,&itemHit);
}

// 

void DoAboutBox(void)
{
	DialogPtr			theDialog;
	EventRecord			dummyEventRecord;
	
	theDialog = GetNewDialog(kAboutBoxDialogResID, nil, (WindowPtr) -1);
	SetPort(theDialog);
	DrawDialog(theDialog);
	
	while(!Button())
	{
		WaitNextEvent(everyEvent, &dummyEventRecord, 30, NULL);
	}
	
	DisposeDialog(theDialog);

	FlushEvents(everyEvent, 0);
}
// 

int doAssembleProgram(WindowPtr windowPtr)
/* To make life easier when updating the avra portion of this program, we will pass
   a series of strings into avra, emulating how that program would run in a 
   command-line interface such as linux.  This means that we have to save the front
   window first, then pass that window's filename to avra.  This isn't how you would
   normally do things on a mac, but what the heck it works. */
{
	docStructureHandle				docStrucHdl;
	AVRFileInfoHandle				AVRFileInfoHdl;
	unsigned char					shortFilePath[255];
	char *							filePath;
	char**							argumentPtr;
	bool							docSaved;
	Str255 							VolName;
	unsigned char 					nameStr[255];
	OSErr 							result;
	short *							volNamePtr;
	int								success;
	
	
	volNamePtr = new short;	
	shortFilePath[0] = '\0';  // Initialise filePath to null so that catdirname will work.
	
	argumentPtr = new char*[20];
	argumentPtr[0] = new char[255];
	argumentPtr[1] = new char[255];
	argumentPtr[2] = new char[255];
	argumentPtr[3] = new char[255];
	filePath = new char[255];

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;

// Save the current document.
	docSaved = doSaveFile(windowPtr);
	
	*volNamePtr = ((*AVRFileInfoHdl)->AVRFSSpec.vRefNum);

	unsigned char numString[10];
	if(docSaved)
	{
	// Get the full path name of the file.
	
		strcpy(argumentPtr[1], "--max_errors");
		NumToString((SInt32) gThePrefs.maxErrors,(unsigned char*)numString);
		P2CStrCpy(argumentPtr[2], numString);
		
		catdirname((char*)shortFilePath, 254, (*AVRFileInfoHdl)->AVRFSSpec.vRefNum, (*AVRFileInfoHdl)->AVRFSSpec.parID);
		result = GetVol(VolName, volNamePtr);
		
		P2CStrCpy(filePath,VolName);		
		strcat(filePath, ":");
		P2CStrCat(filePath, shortFilePath);
		P2CStrCat(filePath, (*AVRFileInfoHdl)->AVRFSSpec.name);
		strcpy(argumentPtr[3],filePath);

		doProgressIndicator(windowPtr);
		setMainStatusText("Assembling...");


		success = avramain(4, argumentPtr);
		
		doDeleteProgressIndicator();
	} else
	{
		doCautionAlert("\pYou have to save the document before assembling it.");		
	}
	
	delete argumentPtr[0];
	delete argumentPtr[1];
	delete argumentPtr[2];
	delete argumentPtr[3];
	delete argumentPtr;
	delete filePath;

	return(success);
}


void doProgramDeviceFromFile(WindowPtr windowPtr)
/* To make life easier when updating the avrp portion of this program, we will pass
   a series of strings into avrp, emulating how that program would run in a 
   command-line interface such as linux.  This means that we have to save the front
   window first, then pass that window's filename to avra.  This isn't how you would
   normally do things on a mac, but what the heck it works. */
{
	unsigned char					shortFilePath[255];
	char *							filePath;
	char *							filePath2;
	char**							argumentPtr;
	char 							portName[200];
	Str255 							VolName;
	unsigned char 					nameStr[255];
	OSErr 							result;
	short *							volNamePtr;
	int 							tempFileNameLength;
	CRMRec 							crmRec;
	CRMRecPtr						crm=&crmRec;
	short							portNumber = 0;	
	StandardFileReply				fileReply;
	SFTypeList						fileTypes;	

	fileTypes[0] = 'TEXT';
	fileTypes[1] = 'BINA';

	StandardGetFile(NULL,2,fileTypes,&fileReply);
	if(!fileReply.sfGood)
	{
		// The user cancelled.
		return;	
	}


	
	crmRec.crmDeviceType=crmSerialDevice;
	crmRec.crmDeviceID=0;
	
	while ((portNumber < gThePrefs.portNumber) && (crm=CRMSearch(crm)))
	{
		CRMSerialPtr portInfo=(CRMSerialPtr)crm->crmAttributes;
		P2CStrCpy(portName, *(portInfo->name));
		portNumber++;
	}

	
	volNamePtr = new short;	
	shortFilePath[0] = '\0';  // Initialise filePath to null so that catdirname will work.
	
	argumentPtr = new char*[20];
	argumentPtr[0] = new char[255];
	argumentPtr[1] = new char[255];
	argumentPtr[2] = new char[255];
	argumentPtr[3] = new char[255];
	argumentPtr[4] = new char[255];
	argumentPtr[5] = new char[255];
	argumentPtr[6] = new char[255];
	argumentPtr[7] = new char[255];
	argumentPtr[8] = new char[255];
	filePath = new char[255];

	windowPtr = FrontWindow();
	
	*volNamePtr = (fileReply.sfFile.vRefNum);

	strcpy(argumentPtr[1], "-s");
	strcpy(argumentPtr[2], portName);
	strcpy(argumentPtr[3], "-a");
	P2CStrCpy(argumentPtr[4], gChipList[gThePrefs.chipType - 1]);
	strcpy(argumentPtr[5], "-f");

		
	catdirname((char*)shortFilePath, 254, fileReply.sfFile.vRefNum, fileReply.sfFile.parID);
	result = GetVol(VolName, volNamePtr);
		
	P2CStrCpy(filePath,VolName);
	strcat(filePath, ":");
	P2CStrCat(filePath, shortFilePath);
	P2CStrCat(filePath, fileReply.sfFile.name);
	strcpy(argumentPtr[6], filePath);
	
	strcpy(argumentPtr[7], "-p");
	strcpy(argumentPtr[8], "-I");


	doProgressIndicator(windowPtr);
	setMainStatusText("Programming");

	avrpmain(9, argumentPtr);
	
	doDeleteProgressIndicator();
	
	delete argumentPtr[0];
	delete argumentPtr[1];
	delete argumentPtr[2];
	delete argumentPtr[3];
	delete argumentPtr[4];
	delete argumentPtr[5];
	delete argumentPtr[6];
	delete argumentPtr[7];
	delete argumentPtr[8];
	delete argumentPtr;
	delete filePath;
//	delete filePath2;

}

void doProgramDevice(WindowPtr windowPtr)
/* To make life easier when updating the avrp portion of this program, we will pass
   a series of strings into avrp, emulating how that program would run in a 
   command-line interface such as linux.  This means that we have to save the front
   window first, then pass that window's filename to avra.  This isn't how you would
   normally do things on a mac, but what the heck it works. */
{
	docStructureHandle				docStrucHdl;
	AVRFileInfoHandle				AVRFileInfoHdl;
	unsigned char					shortFilePath[255];
	char *							filePath;
	char *							filePath2;
	char**							argumentPtr;
	char 							portName[200];
	Str255 							VolName;
	unsigned char 					nameStr[255];
	OSErr 							result;
	short *							volNamePtr;
	int 							tempFileNameLength;
	CRMRec 							crmRec;
	CRMRecPtr						crm=&crmRec;
	short							portNumber = 0;	

	
	crmRec.crmDeviceType=crmSerialDevice;
	crmRec.crmDeviceID=0;
	
	while ((portNumber < gThePrefs.portNumber) && (crm=CRMSearch(crm)))
	{
		CRMSerialPtr portInfo=(CRMSerialPtr)crm->crmAttributes;
		P2CStrCpy(portName, *(portInfo->name));
		portNumber++;
	}

	
	volNamePtr = new short;	
	shortFilePath[0] = '\0';  // Initialise filePath to null so that catdirname will work.
	
	argumentPtr = new char*[20];
	argumentPtr[0] = new char[255];
	argumentPtr[1] = new char[255];
	argumentPtr[2] = new char[255];
	argumentPtr[3] = new char[255];
	argumentPtr[4] = new char[255];
	argumentPtr[5] = new char[255];
	argumentPtr[6] = new char[255];
	argumentPtr[7] = new char[255];
	argumentPtr[8] = new char[255];
	filePath = new char[255];

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;
	
	*volNamePtr = ((*AVRFileInfoHdl)->AVRFSSpec.vRefNum);

	strcpy(argumentPtr[1], "-s");
	strcpy(argumentPtr[2], portName);
	strcpy(argumentPtr[3], "-a");
	P2CStrCpy(argumentPtr[4], gChipList[gThePrefs.chipType - 1]);
	strcpy(argumentPtr[5], "-f");

		
	catdirname((char*)shortFilePath, 254, (*AVRFileInfoHdl)->AVRFSSpec.vRefNum, (*AVRFileInfoHdl)->AVRFSSpec.parID);
	result = GetVol(VolName, volNamePtr);
		
	P2CStrCpy(filePath,VolName);
	strcat(filePath, ":");
	P2CStrCat(filePath, shortFilePath);
	P2CStrCat(filePath, (*AVRFileInfoHdl)->AVRFSSpec.name);
		
	tempFileNameLength = strlen(filePath);
	if((tempFileNameLength > 4) && (strcmp(&filePath[tempFileNameLength - 4], ".asm") == 0))
	{
		filePath[tempFileNameLength - 4] = '\0';
		// This chops off the last 4 characters of argv[1]
	}
	
	strcpy(argumentPtr[6], filePath);
	strcat(argumentPtr[6], ".hex");

	
	strcpy(argumentPtr[7], "-p");
	strcpy(argumentPtr[8], "-I");


	doProgressIndicator(windowPtr);
	setMainStatusText("Programming");
	setStatusText("Programming");

	avrpmain(9, argumentPtr);
	
	doDeleteProgressIndicator();
	
	delete argumentPtr[0];
	delete argumentPtr[1];
	delete argumentPtr[2];
	delete argumentPtr[3];
	delete argumentPtr[4];
	delete argumentPtr[5];
	delete argumentPtr[6];
	delete argumentPtr[7];
	delete argumentPtr[8];
	delete argumentPtr;
	delete filePath;
//	delete filePath2;

}

Boolean IsDialog(WindowPtr windowPtr)
{
	if((gDialogPtr != NULL) || (gPrefsDialogPtr != NULL))
	{ 
		return 1;
	}
	else
	{ 
		return 0;
	}
}

//  eventFilter

pascal Boolean  dialogEventFilter(DialogPtr dialogPtr,EventRecord *eventStrucPtr,SInt16 *itemHit)
{
	Boolean	handledEvent;
	GrafPtr	oldPort;

	handledEvent = false;
	
	if((eventStrucPtr->what == updateEvt) && 
		 ((WindowPtr) eventStrucPtr->message != dialogPtr))
	{
		doUpdate(eventStrucPtr);
	}
	else
	{
		GetPort(&oldPort);
		SetPort(dialogPtr);
		
		handledEvent = StdFilterProc(dialogPtr,eventStrucPtr,itemHit);
		
		SetPort(oldPort);
	}

	return(handledEvent);
}


#ifdef MACOS_800

// Put in here so that Pre MacOS 8.5 systems don't need to link in DriverServicesLib

StringPtr PStrCopy(StringPtr dst,ConstStr255Param src)
{
	BlockMove(src,dst,(src[0] + 1) * sizeof(char));
	return dst;
}

#endif


// Takes a pascal string, turns it into a c string, then concatenates it with 
// the destination C string.
char* P2CStrCat(char* dst,ConstStr255Param src)
{
	unsigned char tempStr[255];
	
	PStrCopy(tempStr, src);
	P2CStr(tempStr);
	strcat(dst, (char*)tempStr);
	return dst;
}

// Takes a pascal string, turns it into a c string, then copies it to
// the destination C string.
char* P2CStrCpy(char* dst,ConstStr255Param src)
{
	unsigned char tempStr[255];
	
	PStrCopy(tempStr, src);
	P2CStr(tempStr);
	strcpy(dst, (char*)tempStr);
	return dst;
}
