// 
// MacAVRpa.c
// 
//
// This program is based on:
// avrp - a programmer for linux, dos etc by Jon Anders Haugum
// avra - an assembler for linux, dos etc by Jon Anders Haugum
// Text1 - a bare bones text editor code base from MacTech magazine
//



// 

//  includes
#include "unistd.mac.h"
#include <DriverServices.h>
#include <TextUtils.h>

#include "macAVRpa.h"
#include "EventSupport.h"
#include "Miscellany.h"
#include "MenuSupport.h"
#include "FileSupport.h"
#include "avra.h"
#include "avrp.h"

//  global variables

ControlActionUPP			scrollActionFunctionUPP;
TEClickLoopUPP				customClickLoopUPP;
ModalFilterUPP				dialogEventFilterUPP;
Boolean						gMacOS85Present = false;
Boolean						gDone;
Boolean						gInBackground;
RgnHandle					gCursorRegion;
SInt16						gNumberOfWindows = 0;
SInt16 						gOldControlValue;


//  main

void  main(void)
{
	OSErr				osError;
	SInt32			response;
	
	// initialise managers

	doInitManagers();

	//  check whether Mac OS 8.5 or later is present

	osError = Gestalt(gestaltSystemVersion,&response);
	
	if(osError == noErr && response >= 0x00000850)
		gMacOS85Present = true;

	//  create routine descriptors

	scrollActionFunctionUPP = NewControlActionProc((ProcPtr) scrollActionFunction);
	customClickLoopUPP = NewTEClickLoopProc((ProcPtr) customClickLoop);
	dialogEventFilterUPP = NewModalFilterProc((ProcPtr) dialogEventFilter);
	
	doSetupEventUPPs();

	//  set up menu bar and menus

	doSetupMenus();
	
	//  install Apple event handlers
	
	doInstallAEHandlers();

	//  enter eventLoop

	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();

	FlushEvents(everyEvent,0);
	
	RegisterAppearanceClient();
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord	eventStructure;
	Boolean			gotEvent;
	SInt32			sleepTime;

	gDone = false;
	gCursorRegion = NewRgn();
	sleepTime = LMGetCaretTime();

	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventStructure,sleepTime,gCursorRegion);

		if(gotEvent)
			doEvents(&eventStructure);
		else
		{
			if(gNumberOfWindows > 0)
				doIdle();
		}
	}
}

//  doIdle

void  doIdle(void)
{
	docStructureHandle	docStrucHdl;
	WindowPtr			windowPtr;
	
	windowPtr = FrontWindow();

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	if(docStrucHdl != NULL)
		TEIdle((*docStrucHdl)->editRecHdl);
}	

//  doKeyEvent

void  doKeyEvent(SInt8 charCode)
{
	WindowPtr						windowPtr;
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	AVRFileInfoHandle				AVRFileInfoHdl;
	SInt16							selectionLength;
	short 							charOffset;
	short 							lineNum;
	Handle							theTextHandle;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;
	AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;

	if(charCode == kTab)
	{
		// Insert a space character...
		TEKey(' ',editRecHdl);
		
		// then keep inserting spaces until the cursor position is a multiple of 3.
		while((currentCharacterPositionInLine(windowPtr) % 3) != 0)
		{
			TEKey(' ',editRecHdl);
		}
		
		doAdjustScrollbar(windowPtr);
		
	}
	else if(charCode == kReturn)
	// This function will indent the next line with the previous one, ala Codewarrior
	{
		lineNum = currentLineNum(windowPtr);
		theTextHandle = (*editRecHdl)->hText;
		
		// Insert a return character...
		TEKey(kReturn,editRecHdl);
		
		charOffset = currentCharacterPositionInLine(windowPtr); 

		// then keep inserting spaces while the previous line has them.
		while((*theTextHandle)[(*editRecHdl)->selStart - charOffset] == ' ')
		{
			TEKey(' ',editRecHdl);
		}
		
		doAdjustScrollbar(windowPtr);
	}
	else if(charCode == kDel)
	{
		selectionLength = doGetSelectLength(editRecHdl);
		if(selectionLength == 0)
			(*editRecHdl)->selEnd += 1;
		TEDelete(editRecHdl);
		doAdjustScrollbar(windowPtr);
	}
	else
	{
		selectionLength = doGetSelectLength(editRecHdl);
		if(((*editRecHdl)->teLength - selectionLength + 1) < kMaxTELength)
		{
			TEKey(charCode,editRecHdl);
			doAdjustScrollbar(windowPtr);
		}
		else
			doErrorAlert(eExceedChara);
	}

	(*AVRFileInfoHdl)->dirty = true;

	doDrawDataPanel(windowPtr);
}

//  scrollActionFunction

pascal void  scrollActionFunction(ControlHandle controlHdl,SInt16 partCode)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							linesToScroll;
	SInt16							controlValue, controlMax;

	windowPtr = (*controlHdl)->contrlOwner;
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	controlValue = GetControlValue(controlHdl);
	controlMax = GetControlMaximum(controlHdl);

	if(partCode)
	{
		if(partCode != kControlIndicatorPart)
		{
			switch(partCode)
			{
				case kControlUpButtonPart:
				case kControlDownButtonPart:
					linesToScroll = 1;
					break;
				
				case kControlPageUpPart:
				case kControlPageDownPart:
					linesToScroll = (((*editRecHdl)->viewRect.bottom - (*editRecHdl)->viewRect.top) /
													 (*editRecHdl)->lineHeight) - 1;
					break;
			}

			if((partCode == kControlDownButtonPart) || (partCode == kControlPageDownPart))
				linesToScroll = -linesToScroll;

			linesToScroll = controlValue - linesToScroll;
			if(linesToScroll < 0)
				linesToScroll = 0;
			else if(linesToScroll > controlMax)
				linesToScroll = controlMax;

			SetControlValue(controlHdl,linesToScroll);

			linesToScroll = controlValue - linesToScroll;
		}
		else
		{
			linesToScroll = gOldControlValue - controlValue;
			gOldControlValue = controlValue;
		}

		if(linesToScroll != 0)
		{
			TEScroll(0,linesToScroll * (*editRecHdl)->lineHeight,editRecHdl);
			doDrawDataPanel(windowPtr);
		}
	}
}

//  doInContent

void  doInContent(EventRecord *eventStrucPtr)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	Point								mouseXY;
	ControlHandle				controlHdl;
	SInt16							partCode;
	Boolean							shiftKeyPosition = false;

	windowPtr = FrontWindow();
	
	if(IsDialogEvent(eventStrucPtr))
	{
	
	} else
	{
		docStrucHdl = (docStructureHandle) GetWRefCon(windowPtr);
		editRecHdl = (*docStrucHdl)->editRecHdl;

		mouseXY = eventStrucPtr->where;
		SetPort(windowPtr);
		GlobalToLocal(&mouseXY);

		if((partCode = FindControl(mouseXY,windowPtr,&controlHdl)) != 0)
		{
			gOldControlValue = GetControlValue(controlHdl);
			TrackControl(controlHdl,mouseXY,scrollActionFunctionUPP);
		}
		else if(PtInRect(mouseXY,&(*editRecHdl)->viewRect))
		{
			if((eventStrucPtr->modifiers & shiftKey) != 0)
				shiftKeyPosition = true;
			TEClick(mouseXY,shiftKeyPosition,editRecHdl);
		}
		
		doDrawDataPanel(windowPtr);
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventStrucPtr)
{
	WindowPtr	windowPtr;

	windowPtr = (WindowPtr) eventStrucPtr->message;

	if(!(IsDialog(windowPtr)))
		doUpdateDocument(windowPtr);
	else
		doUpdateModelessDialog(eventStrucPtr);
}

//  doUpdateDocument

void  doUpdateDocument(WindowPtr windowPtr)
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	GrafPtr							oldPort;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;
	
	GetPort(&oldPort);
	SetPort(windowPtr);

	BeginUpdate(windowPtr);

	TEUpdate(&windowPtr->portRect,editRecHdl);
	UpdateControls(windowPtr,windowPtr->visRgn);

	doDrawDataPanel(windowPtr);

	EndUpdate(windowPtr);

	SetPort(oldPort);
}

//  doUpdateModelessDialog

void  doUpdateModelessDialog(EventRecord *eventStrucPtr)
{
	WindowPtr windowPtr;
	SInt16		itemHit;

	DialogSelect(eventStrucPtr,&windowPtr,&itemHit);
}

//  doActivate

void  doActivate(EventRecord *eventStrucPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventStrucPtr->message;
	becomingActive = ((eventStrucPtr->modifiers & activeFlag) == activeFlag);

//	if(!(IsDialogEvent(eventStrucPtr)))
	if(!IsDialog(windowPtr))
		doActivateDocWindow(windowPtr,becomingActive);
	else
		doActivateModelessDialog(eventStrucPtr,becomingActive);

}

//  doActivateDocWindow

void  doActivateDocWindow(WindowPtr windowPtr,Boolean becomingActive)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	if(becomingActive)
	{
		SetPort(windowPtr);

		(*editRecHdl)->viewRect.bottom = ((((*editRecHdl)->viewRect.bottom -
																				(*editRecHdl)->viewRect.top) / 
																				(*editRecHdl)->lineHeight) * 
																				(*editRecHdl)->lineHeight) + 
																				(*editRecHdl)->viewRect.top;
		(*editRecHdl)->destRect.bottom = (*editRecHdl)->viewRect.bottom;

		TEActivate(editRecHdl);
		ActivateControl((*docStrucHdl)->vScrollbarHdl);
		doAdjustScrollbar(windowPtr);
	}
	else
	{
		TEDeactivate(editRecHdl);
		DeactivateControl((*docStrucHdl)->vScrollbarHdl);
	}
}

//  doActivateModelessDialog

void  doActivateModelessDialog(EventRecord *eventStrucPtr,Boolean becomingActive)
{
	SInt16				whichModeless;
	WindowPtr windowPtr;
	SInt16 itemHit;	

	DialogSelect(eventStrucPtr,&windowPtr,&itemHit);

	if(becomingActive)
	{
		doAdjustMenus();
	}
}

//  doNewDocWindow

WindowPtr  doNewDocWindow(void)
{
	WindowPtr 					windowPtr;
	docStructureHandle			docStrucHdl;
	Rect						destAndViewRect;
	TEHandle					editRecHdl;
	AVRFileInfoHandle			AVRFileInfoHdl;

	if(!(windowPtr = GetNewCWindow(rWindow,NULL,(WindowPtr) -1)))
	{
		doErrorAlert(eWindow);
		return(NULL);
	}

	SetPort(windowPtr);
	TextSize(10);

	if(!(docStrucHdl = (docStructureHandle) NewHandle(sizeof(docStructure))))
	{
		doErrorAlert(eDocStructure);
		return(NULL);
	}
	SetWRefCon(windowPtr,(SInt32) docStrucHdl);

	gNumberOfWindows ++;

	(*docStrucHdl)->vScrollbarHdl = GetNewControl(rVScrollbar,windowPtr);
	
	destAndViewRect = windowPtr->portRect;
	destAndViewRect.right -= 15;
	destAndViewRect.bottom -= 15;
	InsetRect(&destAndViewRect,2,2);

	MoveHHi((Handle) docStrucHdl);
	HLock((Handle) docStrucHdl);

	if(!((*docStrucHdl)->editRecHdl = TENew(&destAndViewRect,&destAndViewRect)))
	{
		DisposeWindow(windowPtr);
		gNumberOfWindows --;
		DisposeHandle((Handle) docStrucHdl);
		doErrorAlert(eEditRecord);
		return(NULL);
	}
	
	HUnlock((Handle) docStrucHdl);
	
	TESetClickLoop(customClickLoopUPP,(*docStrucHdl)->editRecHdl);
	TEAutoView(true,(*docStrucHdl)->editRecHdl);
	TEFeatureFlag(teFOutlineHilite,1,(*docStrucHdl)->editRecHdl);

// Modified for courier

	editRecHdl = (*docStrucHdl)->editRecHdl;
	(*editRecHdl)->txFont = kFontIDCourier;
	
// Set up the file info for that window...
	(*docStrucHdl)->AVRFileInfoHdl = new (AVRFileInfo*);
	AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;
	(*AVRFileInfoHdl) = new AVRFileInfo;
	
	(*AVRFileInfoHdl)->hasBeenSaved = false;
	(*AVRFileInfoHdl)->dirty = true;
	PStrCopy((*AVRFileInfoHdl)->AVRFSSpec.name, "\pUntitled.asm");
	return(windowPtr);
}

//  customClickLoop

pascal Boolean  customClickLoop(void)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	GrafPtr							oldPort;
	RgnHandle						oldClip;
	Rect								tempRect;
	Point								mouseXY;
	SInt16							linesToScroll = 0;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	GetPort(&oldPort);
	SetPort(windowPtr);
	oldClip = NewRgn();
	GetClip(oldClip);
	SetRect(&tempRect,-32767,-32767,32767,32767);
	ClipRect(&tempRect);

	GetMouse(&mouseXY);
	if(mouseXY.v < windowPtr->portRect.top)
	{
		linesToScroll = 1;
		doSetScrollBarValue((*docStrucHdl)->vScrollbarHdl,&linesToScroll);
		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * ((*editRecHdl)->lineHeight),editRecHdl);
	}
	else if(mouseXY.v > windowPtr->portRect.bottom)
	{
		linesToScroll = -1;
		doSetScrollBarValue((*docStrucHdl)->vScrollbarHdl,&linesToScroll);
		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * ((*editRecHdl)->lineHeight),editRecHdl);
	}

	if(linesToScroll != 0)
		doDrawDataPanel(windowPtr);

	SetClip(oldClip);
	DisposeRgn(oldClip);
	SetPort(oldPort);

	return(true);
}

//  doSetScrollBarValue

void  doSetScrollBarValue(ControlHandle controlHdl,SInt16 *linesToScroll)
{
	SInt16	controlValue, controlMax;

	controlValue = GetControlValue(controlHdl);
	controlMax = GetControlMaximum(controlHdl);

	*linesToScroll = controlValue - *linesToScroll;
	if(*linesToScroll < 0)
		*linesToScroll = 0;
	else if(*linesToScroll > controlMax)
		*linesToScroll = controlMax;

	SetControlValue(controlHdl,*linesToScroll);
	*linesToScroll = controlValue - *linesToScroll;
}

//  doGetSelectLength

SInt16  doGetSelectLength(TEHandle editRecHdl)
{
	SInt16	selectionLength;
		
	selectionLength = (*editRecHdl)->selEnd - (*editRecHdl)->selStart;
	return(selectionLength);
}

//  doAdjustScrollbar

void  doAdjustScrollbar(WindowPtr windowPtr)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							numberOfLines, controlMax, controlValue;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	numberOfLines = (*editRecHdl)->nLines;
	if(*(*(*editRecHdl)->hText + (*editRecHdl)->teLength - 1) == kReturn)
		numberOfLines += 1;

	controlMax = numberOfLines - (((*editRecHdl)->viewRect.bottom - 
							 (*editRecHdl)->viewRect.top) /
							 (*editRecHdl)->lineHeight);
	if(controlMax < 0)
		controlMax = 0;
	SetControlMaximum((*docStrucHdl)->vScrollbarHdl,controlMax);

	controlValue = ((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) / 
									(*editRecHdl)->lineHeight;
	if(controlValue < 0)
		controlValue = 0;
	else if(controlValue > controlMax)
		controlValue = controlMax;

	SetControlValue((*docStrucHdl)->vScrollbarHdl,controlValue);

#if TARGET_CPU_PPC
	if(gMacOS85Present)
	{
		SetControlViewSize((*docStrucHdl)->vScrollbarHdl,((*editRecHdl)->viewRect.bottom - 
											 (*editRecHdl)->viewRect.top) / (*editRecHdl)->lineHeight);
	}
#endif
	
	TEScroll(0,((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) - 
					 		(GetControlValue((*docStrucHdl)->vScrollbarHdl) *
							(*editRecHdl)->lineHeight),editRecHdl);
}

//  doAdjustCursor

void  doAdjustCursor(WindowPtr windowPtr)
{
	GrafPtr		oldPort;
	RgnHandle	arrowRegion, iBeamRegion;
	Rect			cursorRect;
	Point			mouseXY;

	if(gInBackground)
	{
		SetCursor(&qd.arrow);
		return;
	}

	GetPort(&oldPort);
	SetPort(windowPtr);

	arrowRegion = NewRgn();
	iBeamRegion = NewRgn();	
	SetRectRgn(arrowRegion,-32768,-32768,32766,32766);

	cursorRect = windowPtr->portRect;
	cursorRect.bottom -= 15;
	cursorRect.right	-= 15;
	LocalToGlobal(&topLeft(cursorRect));
	LocalToGlobal(&botRight(cursorRect));	

	RectRgn(iBeamRegion,&cursorRect);
	DiffRgn(arrowRegion,iBeamRegion,arrowRegion);
	
	GetMouse(&mouseXY);
	LocalToGlobal(&mouseXY);

	if(PtInRgn(mouseXY,iBeamRegion))
	{
		SetCursor(*(GetCursor(iBeamCursor)));
		CopyRgn(iBeamRegion,gCursorRegion);
	}
	else
	{
		SetCursor(&qd.arrow);
		CopyRgn(arrowRegion,gCursorRegion);
	}

	DisposeRgn(arrowRegion);
	DisposeRgn(iBeamRegion);

	SetPort(oldPort);
}

//  doCloseWindow

void  doCloseWindow(WindowPtr windowPtr)
{
	docStructureHandle	docStrucHdl;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;

	DisposeControl((*docStrucHdl)->vScrollbarHdl);
	TEDispose((*docStrucHdl)->editRecHdl);
	delete (*((*docStrucHdl)->AVRFileInfoHdl));
	DisposeHandle((Handle) docStrucHdl);
	DisposeWindow(windowPtr);

	gNumberOfWindows --;
	doAdjustMenus();
}

//  doDrawDataPanel

void  doDrawDataPanel(WindowPtr windowPtr)
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	RGBColor						whiteColour = { 0xFFFF, 0xFFFF, 0xFFFF };					
	RGBColor						blackColour = { 0x0000, 0x0000, 0x0000 };					
	RGBColor						blueColour = { 0x1818, 0x4B4B, 0x8181 };
	ControlHandle					controlHdl;
	Rect							panelRect;
	Str255							textString;
	long							textpos1 = 127, textpos2 = 274, textpos3 = 400;
				
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;
	controlHdl = (*docStrucHdl)->vScrollbarHdl;

	MoveTo(0,282);
	LineTo(495,282);

	RGBForeColor(&whiteColour);
	RGBBackColor(&blueColour);
	SetRect(&panelRect,0,283,495,300);
	EraseRect(&panelRect);

	MoveTo(3,295);
	DrawString("\pNumber of characters =                   Number of Lines =                  Current Line =");

	SetRect(&panelRect,textpos1,284,textpos1 + 30,299);
	EraseRect(&panelRect);	
	SetRect(&panelRect,textpos2,284,textpos2 + 30,299);
	EraseRect(&panelRect);	
	SetRect(&panelRect,textpos3,284,textpos3 + 30,299);
	EraseRect(&panelRect);	

	NumToString((SInt32) (*editRecHdl)->teLength,textString);
	MoveTo(textpos1,295);
	DrawString(textString);

	NumToString((SInt32) (*editRecHdl)->nLines,textString);
	MoveTo(textpos2,295);
	DrawString(textString);

	NumToString((SInt32) currentLineNum(windowPtr),textString);
	MoveTo(textpos3,295);
	DrawString(textString);

	RGBForeColor(&blackColour);
	RGBBackColor(&whiteColour);
}

//  doErrorAlert

void  doErrorAlert(SInt16 errorCode)
{
	AlertStdAlertParamRec	paramRec;
	Str255								errorString;
	SInt16								itemHit;

	paramRec.movable				= true;
	paramRec.helpButton			= false;
	paramRec.filterProc			= NULL;
	paramRec.defaultText		= (StringPtr) kAlertDefaultOKText;
	paramRec.cancelText			= NULL;
	paramRec.otherText			= NULL;
	paramRec.defaultButton	= kAlertStdAlertOKButton;
	paramRec.cancelButton		= 0;
	paramRec.position				= kWindowDefaultPosition;

	GetIndString(errorString,rErrorStrings,errorCode);

	if(errorCode < eWindow)
	{
		StandardAlert(kAlertStopAlert,errorString,NULL,&paramRec,&itemHit);
		ExitToShell();
	}
	else
		StandardAlert(kAlertCautionAlert,errorString,NULL,&paramRec,&itemHit);
}

//  doErrorAlert

void  doCautionAlert(Str255 theAlertString)
{
	AlertStdAlertParamRec	paramRec;
	SInt16								itemHit;

	paramRec.movable				= true;
	paramRec.helpButton			= false;
	paramRec.filterProc			= NULL;
	paramRec.defaultText		= (StringPtr) kAlertDefaultOKText;
	paramRec.cancelText			= NULL;
	paramRec.otherText			= NULL;
	paramRec.defaultButton	= kAlertStdAlertOKButton;
	paramRec.cancelButton		= 0;
	paramRec.position				= kWindowDefaultPosition;

	StandardAlert(kAlertCautionAlert,theAlertString,NULL,&paramRec,&itemHit);
}

//  doErrorAlert

void  doNoteAlert(Str255 theAlertString)
{
	AlertStdAlertParamRec	paramRec;
	SInt16								itemHit;

	paramRec.movable				= true;
	paramRec.helpButton			= false;
	paramRec.filterProc			= NULL;
	paramRec.defaultText		= (StringPtr) kAlertDefaultOKText;
	paramRec.cancelText			= NULL;
	paramRec.otherText			= NULL;
	paramRec.defaultButton	= kAlertStdAlertOKButton;
	paramRec.cancelButton		= 0;
	paramRec.position				= kWindowDefaultPosition;

	StandardAlert(kAlertNoteAlert,theAlertString,NULL,&paramRec,&itemHit);
}

// 

long 	currentLineNum(WindowPtr windowPtr)
// Returns the line number of the currently selected line
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	long 							lineNum = 0;
				
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	while((*editRecHdl)->lineStarts[lineNum] < (*editRecHdl)->selStart)
	{
		lineNum++;
	}

	return lineNum;
}
// 

long 	currentCharacterPositionInLine(WindowPtr windowPtr)
// Returns the line number of the currently selected line
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	long 							lineNum, charNum;
				
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	lineNum = currentLineNum(windowPtr);
	
	charNum = (*editRecHdl)->selStart - (*editRecHdl)->lineStarts[lineNum - 1];

	return charNum;
}
// 

void DoAboutBox(void)
{
	DialogPtr			theDialog;
	
	theDialog = GetNewDialog(kAboutBoxDialogResID, nil, (WindowPtr) -1);
	SetPort(theDialog);
	DrawDialog(theDialog);
	
	while(!Button())
	{
		WaitNextEvent(everyEvent, NULL, 30, NULL);
	}
	
	DisposeDialog(theDialog);

	FlushEvents(everyEvent, 0);
}
// 

void doAssembleProgram(WindowPtr windowPtr)
/* To make life easier when updating the avra portion of this program, we will pass
   a series of strings into avra, emulating how that program would run in a 
   command-line interface such as linux.  This means that we have to save the front
   window first, then pass that window's filename to avra.  This isn't how you would
   normally do things on a mac, but what the heck it works. */
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	AVRFileInfoHandle				AVRFileInfoHdl;
	unsigned char					shortFilePath[255];
	char *							filePath;
	char *							filePath2;
	char**							argumentPtr;
	bool							docSaved;
	Str255 							VolName;
	unsigned char 					nameStr[255];
	OSErr 							result;
	short *							volNamePtr;
	
	
	volNamePtr = new short;	
	shortFilePath[0] = '\0';  // Initialise filePath to null so that catdirname will work.
	
	argumentPtr = new char*[20];
	argumentPtr[0] = new char[255];
	argumentPtr[1] = new char[255];
	argumentPtr[2] = new char[255];

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;

// Save the current document.
	docSaved = doSaveFile(windowPtr);
	
	*volNamePtr = ((*AVRFileInfoHdl)->AVRFSSpec.vRefNum);


	if(docSaved)
	{
	// Get the full path name of the file.
		
		catdirname((char*)shortFilePath, 254, (*AVRFileInfoHdl)->AVRFSSpec.vRefNum, (*AVRFileInfoHdl)->AVRFSSpec.parID);
		result = GetVol(VolName, volNamePtr);
		
		P2CStr(VolName);
		strcpy(filePath,(char*)VolName);		
		strcat(filePath, ":");
		P2CStr(shortFilePath);
		strcat(filePath, (char*)shortFilePath);
		PStrCopy(nameStr, (*AVRFileInfoHdl)->AVRFSSpec.name);
		P2CStr(nameStr);
		strcat(filePath, (char*)nameStr);
		strcpy(argumentPtr[1],filePath);

		doProgressIndicator(windowPtr);

		avramain(2, argumentPtr);
		
		doDeleteProgressIndicator();
	} else
	{
		doCautionAlert("\pYou have to save the document before assembling it.");		
	}
	
	delete argumentPtr[0];
	delete argumentPtr[1];
	delete argumentPtr[2];
	delete argumentPtr;

}

void doProgramDevice(WindowPtr windowPtr)
/* To make life easier when updating the avrp portion of this program, we will pass
   a series of strings into avrp, emulating how that program would run in a 
   command-line interface such as linux.  This means that we have to save the front
   window first, then pass that window's filename to avra.  This isn't how you would
   normally do things on a mac, but what the heck it works. */
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	AVRFileInfoHandle				AVRFileInfoHdl;
	unsigned char					shortFilePath[255];
	char *							filePath;
	char *							filePath2;
	char**							argumentPtr;
	bool							docSaved;
	Str255 							VolName;
	unsigned char 					nameStr[255];
	OSErr 							result;
	short *							volNamePtr;
	int tempFileNameLength;
	
	
	volNamePtr = new short;	
	shortFilePath[0] = '\0';  // Initialise filePath to null so that catdirname will work.
	
	argumentPtr = new char*[20];
	argumentPtr[0] = new char[255];
	argumentPtr[1] = new char[255];
	argumentPtr[2] = new char[255];
	argumentPtr[3] = new char[255];
	argumentPtr[4] = new char[255];
	argumentPtr[5] = new char[255];
	argumentPtr[6] = new char[255];
	argumentPtr[7] = new char[255];
	argumentPtr[8] = new char[255];
	filePath = new char[255];
	filePath2 = new char[255];

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;
	
	*volNamePtr = ((*AVRFileInfoHdl)->AVRFSSpec.vRefNum);

	strcpy(argumentPtr[1], "-s");
	strcpy(argumentPtr[2], "printer");
	strcpy(argumentPtr[3], "-a");
	strcpy(argumentPtr[4], "AT90S1200-C");
	strcpy(argumentPtr[5], "-f");

		
	catdirname((char*)shortFilePath, 254, (*AVRFileInfoHdl)->AVRFSSpec.vRefNum, (*AVRFileInfoHdl)->AVRFSSpec.parID);
	result = GetVol(VolName, volNamePtr);
	
	P2CStr(VolName);
	strcpy(filePath,(char*)VolName);		
	strcat(filePath, ":");
	P2CStr(shortFilePath);
	strcat(filePath, (char*)shortFilePath);
	PStrCopy(nameStr, (*AVRFileInfoHdl)->AVRFSSpec.name);
	P2CStr(nameStr);
	strcat(filePath, (char*)nameStr);
		
	tempFileNameLength = strlen(filePath);
	if((tempFileNameLength > 4) && (strcmp(&filePath[tempFileNameLength - 4], ".asm") == 0))
	{
		filePath[tempFileNameLength - 4] = '\0';
		// This chops off the last 4 characters of argv[1]
	}
	
	strcpy(argumentPtr[6], filePath);
	strcat(argumentPtr[6], ".hex");

	
	strcpy(argumentPtr[7], "-p");
	strcpy(argumentPtr[8], "-I");


//	doProgressIndicator(windowPtr);

	avrpmain(9, argumentPtr);
	
//	doDeleteProgressIndicator();
	
	delete argumentPtr[0];
	delete argumentPtr[1];
	delete argumentPtr[2];
	delete argumentPtr[3];
	delete argumentPtr[4];
	delete argumentPtr[5];
	delete argumentPtr[6];
	delete argumentPtr[7];
	delete argumentPtr[8];
	delete argumentPtr;
	delete filePath;
	delete filePath2;

}

Boolean IsDialog(WindowPtr windowPtr)
{
	if((gDialogPtr != NULL) || (gPrefsDialogPtr != NULL))
	{ 
		return 1;
	}
	else
	{ 
		return 0;
	}
}

//  eventFilter

pascal Boolean  dialogEventFilter(DialogPtr dialogPtr,EventRecord *eventStrucPtr,SInt16 *itemHit)
{
	Boolean	handledEvent;
	GrafPtr	oldPort;

	handledEvent = false;
	
	if((eventStrucPtr->what == updateEvt) && 
		 ((WindowPtr) eventStrucPtr->message != dialogPtr))
	{
		doUpdate(eventStrucPtr);
	}
	else
	{
		GetPort(&oldPort);
		SetPort(dialogPtr);

		handledEvent = StdFilterProc(dialogPtr,eventStrucPtr,itemHit);

		SetPort(oldPort);
	}

	return(handledEvent);
}



	
