#include "MacAVRpa.h"
#include "MenuSupport.h"
#include "FileSupport.h"
#include "Preferences.h"
#include "TextWindowSupport.h"

void doSetupMenus(void)
{	
	Handle				menubarHdl;
	MenuHandle			menuHdl;
	OSErr				osErr;	

	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		doErrorAlert(eMenuBar);
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		doErrorAlert(eMenu);
	else
		AppendResMenu(menuHdl,'DRVR');

	osErr = HMGetHelpMenuHandle(&menuHdl);
	if(osErr == noErr)
		AppendMenu(menuHdl,"\pText1 Help");
	else
		doErrorAlert(eMenu);
		
	doAdjustMenus();

}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle					fileMenuHdl, editMenuHdl, programMenuHdl, serialPortHdl;
	WindowPtr					windowPtr;
	docStructureHandle			docStrucHdl;
	TEHandle					editRecHdl;
	AVRFileInfoHandle			AVRFileInfoHdl;
	SInt32						scrapOffset;

	fileMenuHdl = GetMenuHandle(mFile);
	editMenuHdl = GetMenuHandle(mEdit);
	programMenuHdl = GetMenuHandle(mProgram);
	serialPortHdl = GetMenuHandle(mSerialPort);
	
	if(gNumberOfWindows > 0)
	{
		windowPtr = FrontWindow();
		
		if(IsDialog(windowPtr))
		{
		
		} else
		{
			docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
			editRecHdl = (*docStrucHdl)->editRecHdl;
			AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;

			EnableItem(fileMenuHdl,iClose);
			EnableItem(programMenuHdl, 0);

			if((*editRecHdl)->selStart < (*editRecHdl)->selEnd)
			{
				EnableItem(editMenuHdl,iCut);
				EnableItem(editMenuHdl,iCopy);
				EnableItem(editMenuHdl,iClear);
			}
			else
			{
				DisableItem(editMenuHdl,iCut);
				DisableItem(editMenuHdl,iCopy);
				DisableItem(editMenuHdl,iClear);
			}

			if(GetScrap(NULL,'TEXT',&scrapOffset) > 0)
				EnableItem(editMenuHdl,iPaste);
			else
				DisableItem(editMenuHdl,iPaste);
				
			if((*editRecHdl)->teLength > 0)
			{
				EnableItem(fileMenuHdl,iSaveAs);
				EnableItem(editMenuHdl,iSelectAll);
				if(((*AVRFileInfoHdl)->hasBeenSaved) && (*AVRFileInfoHdl)->dirty)
				{
					EnableItem(fileMenuHdl, iSave);
					EnableItem(fileMenuHdl, iRevert);
				} else
				{
					DisableItem(fileMenuHdl, iSave);
					DisableItem(fileMenuHdl, iRevert);
				}
			}
			else
			{
				DisableItem(fileMenuHdl, iSave);
				DisableItem(fileMenuHdl,iSaveAs);
				DisableItem(editMenuHdl,iSelectAll);
			}
		}
	}
	else
	{
		EnableItem(fileMenuHdl, 0);
		
		EnableItem(fileMenuHdl, iOpen);
		EnableItem(fileMenuHdl, iQuit);
		DisableItem(fileMenuHdl, iClose);
		DisableItem(fileMenuHdl, iSave);
		DisableItem(fileMenuHdl, iSaveAs);
		DisableItem(fileMenuHdl, iRevert);
		
		EnableItem(editMenuHdl, 0);
		DisableItem(editMenuHdl, iClear);
		DisableItem(editMenuHdl, iSelectAll);
		
		DisableItem(programMenuHdl, 0);
	}		

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16  menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID	 = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
			{
				DoAboutBox();
			}
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			doFileMenu(menuItem);
			break;

		case mEdit:
			doEditMenu(menuItem);
			break;

		case mProgram:
			doProgramMenu(menuItem);
			break;

		case kHMHelpMenuID:
			if(FrontWindow())	
				doActivateDocWindow(FrontWindow(),false);
			doHelpMenu(menuItem);
			break;
	}
	doAdjustMenus();
	HiliteMenu(0);
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	WindowPtr						windowPtr;

	switch(menuItem)
	{
		case iNew:
			if(windowPtr = doNewDocWindow())
				ShowWindow(windowPtr);
			break;

		case iOpen:
			doOpenCommand();
			break;

		case iClose:
			doCloseWindow(FrontWindow());
			break;

		case iSave:
			doSaveFile(FrontWindow());
			break;

		case iSaveAs:
			doSaveAsFile(FrontWindow());
			break;

		case iRevert:
			doRevert(FrontWindow());
			break;

		case iQuit:
			gDone = true;
			break;
	}
}

//  doEditMenu

void	doEditMenu(SInt16 menuItem)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt32							totalSize, contigSize, newSize, scrapOffset;
	SInt16							selectionLength;

	windowPtr = FrontWindow();
	if(windowPtr)
	{
		docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
		editRecHdl = (*docStrucHdl)->editRecHdl;
	}

	switch(menuItem)
	{
		case iUndo:
			break;

		case iCut:
			if(ZeroScrap() == noErr)
			{
				PurgeSpace(&totalSize,&contigSize);
				selectionLength = doGetSelectLength(editRecHdl);
				if(selectionLength > contigSize)
					doErrorAlert(eNoSpaceCut);
				else
				{
					TECut(editRecHdl);
					doAdjustScrollbar(windowPtr);
					if(TEToScrap() != noErr)
						ZeroScrap();
				}
			}
			break;

		case iCopy:
			if(ZeroScrap() == noErr)
			{
				TECopy(editRecHdl);
				if(TEToScrap() != noErr)
					ZeroScrap();
			}
			break;

		case iPaste:
			newSize = (*editRecHdl)->teLength + GetScrap(NULL,'TEXT',&scrapOffset);
			if(newSize > kMaxTELength)
				doErrorAlert(eNoSpacePaste);
			else
			{
				if(TEFromScrap() == noErr)
				{
					TEPaste(editRecHdl);
					doAdjustScrollbar(windowPtr);
				}
			}
			break;

		case iClear:
			TEDelete(editRecHdl);
			doAdjustScrollbar(windowPtr);
			break;

		case iSelectAll:
			TESetSelect(0,(*editRecHdl)->teLength,editRecHdl);
			break;
			
		case iPreferences:
			doPreferences(windowPtr);
			break;

	}

	doDrawDataPanel(windowPtr);
}

//  doEditMenu

void	doProgramMenu(SInt16 menuItem)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	switch(menuItem)
	{
		case iAssemble:
			doAssembleProgram(windowPtr);
			break;

		case iAssembleAndProgram:
			if(!doAssembleProgram(windowPtr))
				doProgramDevice(windowPtr);
			break;

		case iProgramNoAssemble:
			doProgramDeviceFromFile(windowPtr);
			break;

		case iVerifyFlash:
			break;

		case iLockChip:
			break;

		case iSetFuses:
			break;
	}
}

//  doHelpMenu

void	doHelpMenu(SInt16 menuItem)
{
	MenuHandle	helpMenuHdl;
	SInt16			origHelpItems, numItems;

	HMGetHelpMenuHandle(&helpMenuHdl);

	numItems = CountMenuItems(helpMenuHdl);
	origHelpItems = numItems - 1;

	if(menuItem > origHelpItems)
		doHelp();	
}

