// 
// Text1.c
// 
//
// This program demonstrates:
//
// 	A "bare-bones" monostyled text editor.
//
// 	A Help dialog which features the integrated scrolling of multistyled text and 
//		pictures.
//
// In the monostyled text editor demonstration, a panel is displayed at the bottom of all
// opened windows.  This panel displays the edit record length, number of lines, line 
// height, destination rectangle (top), scroll bar value, and scroll bar maximum value. 
//
// The bulk of the source code for the Help dialog is contained in the file HelpDialog.c.
// The dialog itself displays information intended to assist the user in adapting the
// Help dialog source code and resources to the requirements of his/her own application.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit, and Help dialog
//		pop-up menus (preload, non-purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially visible).  
//
// 	'CNTL' resources (purgeable) for the vertical scroll bars in the text editor window
//		and Help dialog, and for the pop-up menu in the Help Dialog.
//     
// 	A 'DLOG' resource (purgeable, initially invisible) and associated 'dctb' resource 
//		(purgeable) for the Help dialog.
//
// 	'DITL' resources (purgeable) for the 'ALRT' and 'DLOG' resources.
//
// 	'TEXT' and associated 'styl' resources (all purgeable) for the Help dialog.
//
// 	'PICT' resources (purgeable) for the Help dialog.
//
// 	A 'STR ' resource  (purgeable) containing error text strings.  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible	flags set.	  
//
// 

//  includes

#include <Appearance.h>
#include <Balloons.h>	
//#include <ControlDefinitions.h>
#include <Devices.h>
#include <Gestalt.h>
#include <LowMem.h>
#include <Scrap.h>
#include <Sound.h>
#include <StandardFile.h>
#include <ToolUtils.h>

//  defines

#define rMenubar				129
#define mApple					128
#define  iAbout					1
#define mFile						129
#define  iNew						1
#define  iOpen					2
#define  iClose					4
#define  iSaveAs				6
#define  iQuit					12
#define mEdit						130
#define  iUndo					1
#define  iCut						3
#define  iCopy					4
#define  iPaste					5
#define  iClear					6
#define  iSelectAll			7
#define mProgram				131
#define  iAssemble				1
#define  iAssembleAndProgram	2
#define  iProgramNoAssemble		3
#define  iVerifyFlash			4
#define  iLockChip				6
#define  iSetFuses				7
#define rWindow					128
#define rVScrollbar			128
#define rErrorStrings		128
#define  eMenuBar				1
#define  eMenu					2
#define  eWindow				3
#define  eDocStructure	4
#define  eEditRecord		5
#define  eExceedChara		6
#define  eNoSpaceCut		7
#define  eNoSpacePaste	8
#define  eProgramming		13
#define kMaxTELength		32767
#define kTab						0x09
#define kDel						0x7F
#define kReturn					0x0D
#define topLeft(r)			(((Point *) &(r))[0])
#define botRight(r)			(((Point *) &(r))[1])
#define kAboutBoxDialogResID	129

//  typedefs

typedef struct
{
	TEHandle			editRecHdl;
	ControlHandle vScrollbarHdl;
} docStructure, **docStructureHandle;

//  global variables

ControlActionUPP	scrollActionFunctionUPP;
TEClickLoopUPP		customClickLoopUPP;
Boolean						gMacOS85Present = false;
Boolean						gDone;
Boolean						gInBackground;
RgnHandle					gCursorRegion;
SInt16						gNumberOfWindows = 0;
SInt16 						gOldControlValue;

//  function prototypes

void						main								(void);
void						doInitManagers						(void);
void						eventLoop							(void);
void						doIdle								(void);
void						doEvents							(EventRecord *);
void						doKeyEvent							(SInt8);
pascal void  				scrollActionFunction				(ControlHandle,SInt16);
void						doInContent							(EventRecord *);
void						doUpdate							(EventRecord *);
void						doActivate							(EventRecord *);
void						doActivateDocWindow					(WindowPtr,Boolean);
void						doOSEvent							(EventRecord *);
WindowPtr					doNewDocWindow						(void);
pascal Boolean				customClickLoop						(void);
void						doSetScrollBarValue					(ControlHandle,SInt16 *);
void						doAdjustMenus						(void);
void						doMenuChoice						(SInt32);
void						doFileMenu							(SInt16);
void						doEditMenu							(SInt16);
SInt16						doGetSelectLength					(TEHandle);
void						doAdjustScrollbar					(WindowPtr);
void						doAdjustCursor						(WindowPtr);
void						doCloseWindow						(WindowPtr);
void						doSaveAsFile						(TEHandle);
void						doOpenCommand						(void);
void						doOpenFile							(FSSpec);
void						doDrawDataPanel						(WindowPtr);
void						doErrorAlert						(SInt16);
void						doHelpMenu							(SInt16);

extern void					doHelp								(void);

long 						currentLineNum						(WindowPtr windowPtr);
long 						currentCharacterPositionInLine		(WindowPtr windowPtr);
void 						DoAboutBox							(void);

//  main

void  main(void)
{
	OSErr				osError;
	SInt32			response;
	Handle			menubarHdl;
	MenuHandle	menuHdl;
	OSErr				osErr;	
	
	// initialise managers

	doInitManagers();

	//  check whether Mac OS 8.5 or later is present

	osError = Gestalt(gestaltSystemVersion,&response);
	
	if(osError == noErr && response >= 0x00000850)
		gMacOS85Present = true;

	//  create routine descriptors

	scrollActionFunctionUPP = NewControlActionProc((ProcPtr) scrollActionFunction);
	customClickLoopUPP = NewTEClickLoopProc((ProcPtr) customClickLoop);

	//  set up menu bar and menus
	
	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		doErrorAlert(eMenuBar);
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		doErrorAlert(eMenu);
	else
		AppendResMenu(menuHdl,'DRVR');

	osErr = HMGetHelpMenuHandle(&menuHdl);
	if(osErr == noErr)
		AppendMenu(menuHdl,"\pText1 Help");
	else
		doErrorAlert(eMenu);

	//  open an untitled window

	doNewDocWindow();

	//  enter eventLoop

	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();

	FlushEvents(everyEvent,0);
	
	RegisterAppearanceClient();
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord	eventStructure;
	Boolean			gotEvent;
	SInt32			sleepTime;

	gDone = false;
	gCursorRegion = NewRgn();
	sleepTime = LMGetCaretTime();

	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventStructure,sleepTime,gCursorRegion);

		if(gotEvent)
			doEvents(&eventStructure);
		else
		{
			if(gNumberOfWindows > 0)
				doIdle();
		}
	}
}

//  doIdle

void  doIdle(void)
{
	docStructureHandle	docStrucHdl;
	WindowPtr			windowPtr;
	
	windowPtr = FrontWindow();

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	if(docStrucHdl != NULL)
		TEIdle((*docStrucHdl)->editRecHdl);
}	

//  doEvents

void	doEvents(EventRecord *eventStrucPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;
	SInt8			charCode;

	switch(eventStrucPtr->what)
	{
		case mouseDown:
			partCode = FindWindow(eventStrucPtr->where,&windowPtr);
			switch(partCode)
			{
				case inMenuBar:
					doAdjustMenus();
					doMenuChoice(MenuSelect(eventStrucPtr->where));
					break;

				case inContent:
					if(windowPtr != FrontWindow())
						SelectWindow(windowPtr);
					else
						doInContent(eventStrucPtr);
					break;

				case inDrag:
					DragWindow(windowPtr,eventStrucPtr->where,&qd.screenBits.bounds);
					doAdjustCursor(windowPtr);
					break;

				case inGoAway:
					if(TrackGoAway(windowPtr,eventStrucPtr->where))
						doCloseWindow(FrontWindow());
					break;
			}
			break;

		case keyDown:
		case autoKey:
			charCode = eventStrucPtr->message & charCodeMask;
			if((eventStrucPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuEvent(eventStrucPtr));
			}
			else
				doKeyEvent(charCode);
			break;

		case updateEvt:
			doUpdate(eventStrucPtr);
			break;

		case activateEvt:
			doActivate(eventStrucPtr);
			break;

		case osEvt:
			doOSEvent(eventStrucPtr);
			break;
	}
}

//  doKeyEvent

void  doKeyEvent(SInt8 charCode)
{
	WindowPtr						windowPtr;
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							selectionLength;
	short 							charOffset;
	short 							lineNum;
	Handle							theTextHandle;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	if(charCode == kTab)
	{
		// Insert a space character...
		TEKey(' ',editRecHdl);
		
		// then keep inserting spaces until the cursor position is a multiple of 3.
		while((currentCharacterPositionInLine(windowPtr) % 3) != 0)
		{
			TEKey(' ',editRecHdl);
		}
		
		doAdjustScrollbar(windowPtr);
		
	}
	else if(charCode == kReturn)
	// This function will indent the next line with the previous one, ala Codewarrior
	{
		lineNum = currentLineNum(windowPtr);
		theTextHandle = (*editRecHdl)->hText;
		
		// Insert a return character...
		TEKey(kReturn,editRecHdl);
		
		charOffset = currentCharacterPositionInLine(windowPtr); 

		// then keep inserting spaces while the previous line has them.
		while((*theTextHandle)[(*editRecHdl)->selStart - charOffset] == ' ')
		{
			TEKey(' ',editRecHdl);
		}
		
		doAdjustScrollbar(windowPtr);
	}
	else if(charCode == kDel)
	{
		selectionLength = doGetSelectLength(editRecHdl);
		if(selectionLength == 0)
			(*editRecHdl)->selEnd += 1;
		TEDelete(editRecHdl);
		doAdjustScrollbar(windowPtr);
	}
	else
	{
		selectionLength = doGetSelectLength(editRecHdl);
		if(((*editRecHdl)->teLength - selectionLength + 1) < kMaxTELength)
		{
			TEKey(charCode,editRecHdl);
			doAdjustScrollbar(windowPtr);
		}
		else
			doErrorAlert(eExceedChara);
	}

	doDrawDataPanel(windowPtr);
}

//  scrollActionFunction

pascal void  scrollActionFunction(ControlHandle controlHdl,SInt16 partCode)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							linesToScroll;
	SInt16							controlValue, controlMax;

	windowPtr = (*controlHdl)->contrlOwner;
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	controlValue = GetControlValue(controlHdl);
	controlMax = GetControlMaximum(controlHdl);

	if(partCode)
	{
		if(partCode != kControlIndicatorPart)
		{
			switch(partCode)
			{
				case kControlUpButtonPart:
				case kControlDownButtonPart:
					linesToScroll = 1;
					break;
				
				case kControlPageUpPart:
				case kControlPageDownPart:
					linesToScroll = (((*editRecHdl)->viewRect.bottom - (*editRecHdl)->viewRect.top) /
													 (*editRecHdl)->lineHeight) - 1;
					break;
			}

			if((partCode == kControlDownButtonPart) || (partCode == kControlPageDownPart))
				linesToScroll = -linesToScroll;

			linesToScroll = controlValue - linesToScroll;
			if(linesToScroll < 0)
				linesToScroll = 0;
			else if(linesToScroll > controlMax)
				linesToScroll = controlMax;

			SetControlValue(controlHdl,linesToScroll);

			linesToScroll = controlValue - linesToScroll;
		}
		else
		{
			linesToScroll = gOldControlValue - controlValue;
			gOldControlValue = controlValue;
		}

		if(linesToScroll != 0)
		{
			TEScroll(0,linesToScroll * (*editRecHdl)->lineHeight,editRecHdl);
			doDrawDataPanel(windowPtr);
		}
	}
}

//  doInContent

void  doInContent(EventRecord *eventStrucPtr)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	Point								mouseXY;
	ControlHandle				controlHdl;
	SInt16							partCode;
	Boolean							shiftKeyPosition = false;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) GetWRefCon(windowPtr);
	editRecHdl = (*docStrucHdl)->editRecHdl;

	mouseXY = eventStrucPtr->where;
	SetPort(windowPtr);
	GlobalToLocal(&mouseXY);

	if((partCode = FindControl(mouseXY,windowPtr,&controlHdl)) != 0)
	{
		gOldControlValue = GetControlValue(controlHdl);
		TrackControl(controlHdl,mouseXY,scrollActionFunctionUPP);
	}
	else if(PtInRect(mouseXY,&(*editRecHdl)->viewRect))
	{
		if((eventStrucPtr->modifiers & shiftKey) != 0)
			shiftKeyPosition = true;
		TEClick(mouseXY,shiftKeyPosition,editRecHdl);
	}
	
	doDrawDataPanel(windowPtr);
}

//  doUpdate

void	doUpdate(EventRecord *eventStrucPtr)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	GrafPtr							oldPort;

	windowPtr = (WindowPtr) eventStrucPtr->message;
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;
	
	GetPort(&oldPort);
	SetPort(windowPtr);

	BeginUpdate((WindowPtr)eventStrucPtr->message);

	TEUpdate(&windowPtr->portRect,editRecHdl);
	UpdateControls(windowPtr,windowPtr->visRgn);

	doDrawDataPanel(windowPtr);

	EndUpdate((WindowPtr)eventStrucPtr->message);

	SetPort(oldPort);
}

//  doActivate

void  doActivate(EventRecord *eventStrucPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventStrucPtr->message;
	becomingActive = ((eventStrucPtr->modifiers & activeFlag) == activeFlag);
	doActivateDocWindow(windowPtr,becomingActive);
}

//  doActivateDocWindow

void  doActivateDocWindow(WindowPtr windowPtr,Boolean becomingActive)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	if(becomingActive)
	{
		SetPort(windowPtr);

		(*editRecHdl)->viewRect.bottom = ((((*editRecHdl)->viewRect.bottom -
																				(*editRecHdl)->viewRect.top) / 
																				(*editRecHdl)->lineHeight) * 
																				(*editRecHdl)->lineHeight) + 
																				(*editRecHdl)->viewRect.top;
		(*editRecHdl)->destRect.bottom = (*editRecHdl)->viewRect.bottom;

		TEActivate(editRecHdl);
		ActivateControl((*docStrucHdl)->vScrollbarHdl);
		doAdjustScrollbar(windowPtr);
	}
	else
	{
		TEDeactivate(editRecHdl);
		DeactivateControl((*docStrucHdl)->vScrollbarHdl);
	}
}

//  doOSEvent

void	doOSEvent(EventRecord *eventStrucPtr)
{
	switch((eventStrucPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			gInBackground = (eventStrucPtr->message & resumeFlag) == 0;
			if(!gInBackground)
				SetCursor(&qd.arrow);
			if(gNumberOfWindows > 0)
				doActivateDocWindow(FrontWindow(),!gInBackground);
			HiliteMenu(0);
			break;

		case mouseMovedMessage:
			doAdjustCursor(FrontWindow());
			break;
	}
}

//  doNewDocWindow

WindowPtr  doNewDocWindow(void)
{
	WindowPtr 					windowPtr;
	docStructureHandle			docStrucHdl;
	Rect								destAndViewRect;
	TEHandle						editRecHdl;

	if(!(windowPtr = GetNewCWindow(rWindow,NULL,(WindowPtr) -1)))
	{
		doErrorAlert(eWindow);
		return(NULL);
	}

	SetPort(windowPtr);
	TextSize(10);

	if(!(docStrucHdl = (docStructureHandle) NewHandle(sizeof(docStructure))))
	{
		doErrorAlert(eDocStructure);
		return(NULL);
	}
	SetWRefCon(windowPtr,(SInt32) docStrucHdl);

	gNumberOfWindows ++;

	(*docStrucHdl)->vScrollbarHdl = GetNewControl(rVScrollbar,windowPtr);
	
	destAndViewRect = windowPtr->portRect;
	destAndViewRect.right -= 15;
	destAndViewRect.bottom -= 15;
	InsetRect(&destAndViewRect,2,2);

	MoveHHi((Handle) docStrucHdl);
	HLock((Handle) docStrucHdl);

	if(!((*docStrucHdl)->editRecHdl = TENew(&destAndViewRect,&destAndViewRect)))
	{
		DisposeWindow(windowPtr);
		gNumberOfWindows --;
		DisposeHandle((Handle) docStrucHdl);
		doErrorAlert(eEditRecord);
		return(NULL);
	}
	
	HUnlock((Handle) docStrucHdl);
	
	TESetClickLoop(customClickLoopUPP,(*docStrucHdl)->editRecHdl);
	TEAutoView(true,(*docStrucHdl)->editRecHdl);
	TEFeatureFlag(teFOutlineHilite,1,(*docStrucHdl)->editRecHdl);

// Modified for courier

	editRecHdl = (*docStrucHdl)->editRecHdl;
	(*editRecHdl)->txFont = kFontIDCourier;

	return(windowPtr);
}

//  customClickLoop

pascal Boolean  customClickLoop(void)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	GrafPtr							oldPort;
	RgnHandle						oldClip;
	Rect								tempRect;
	Point								mouseXY;
	SInt16							linesToScroll = 0;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	GetPort(&oldPort);
	SetPort(windowPtr);
	oldClip = NewRgn();
	GetClip(oldClip);
	SetRect(&tempRect,-32767,-32767,32767,32767);
	ClipRect(&tempRect);

	GetMouse(&mouseXY);
	if(mouseXY.v < windowPtr->portRect.top)
	{
		linesToScroll = 1;
		doSetScrollBarValue((*docStrucHdl)->vScrollbarHdl,&linesToScroll);
		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * ((*editRecHdl)->lineHeight),editRecHdl);
	}
	else if(mouseXY.v > windowPtr->portRect.bottom)
	{
		linesToScroll = -1;
		doSetScrollBarValue((*docStrucHdl)->vScrollbarHdl,&linesToScroll);
		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * ((*editRecHdl)->lineHeight),editRecHdl);
	}

	if(linesToScroll != 0)
		doDrawDataPanel(windowPtr);

	SetClip(oldClip);
	DisposeRgn(oldClip);
	SetPort(oldPort);

	return(true);
}

//  doSetScrollBarValue

void  doSetScrollBarValue(ControlHandle controlHdl,SInt16 *linesToScroll)
{
	SInt16	controlValue, controlMax;

	controlValue = GetControlValue(controlHdl);
	controlMax = GetControlMaximum(controlHdl);

	*linesToScroll = controlValue - *linesToScroll;
	if(*linesToScroll < 0)
		*linesToScroll = 0;
	else if(*linesToScroll > controlMax)
		*linesToScroll = controlMax;

	SetControlValue(controlHdl,*linesToScroll);
	*linesToScroll = controlValue - *linesToScroll;
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle					fileMenuHdl, editMenuHdl;
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt32							scrapOffset;

	fileMenuHdl = GetMenuHandle(mFile);
	editMenuHdl = GetMenuHandle(mEdit);

	if(gNumberOfWindows > 0)
	{
		windowPtr = FrontWindow();
		docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
		editRecHdl = (*docStrucHdl)->editRecHdl;

		EnableItem(fileMenuHdl,iClose);

		if((*editRecHdl)->selStart < (*editRecHdl)->selEnd)
		{
			EnableItem(editMenuHdl,iCut);
			EnableItem(editMenuHdl,iCopy);
			EnableItem(editMenuHdl,iClear);
		}
		else
		{
			DisableItem(editMenuHdl,iCut);
			DisableItem(editMenuHdl,iCopy);
			DisableItem(editMenuHdl,iClear);
		}

		if(GetScrap(NULL,'TEXT',&scrapOffset) > 0)
			EnableItem(editMenuHdl,iPaste);
		else
			DisableItem(editMenuHdl,iPaste);
			
		if((*editRecHdl)->teLength > 0)
		{
			EnableItem(fileMenuHdl,iSaveAs);
			EnableItem(editMenuHdl,iSelectAll);
		}
		else
		{
			DisableItem(fileMenuHdl,iSaveAs);
			DisableItem(editMenuHdl,iSelectAll);
		}
	}
	else
	{
		DisableItem(fileMenuHdl,iClose);
		DisableItem(fileMenuHdl,iSaveAs);
		DisableItem(editMenuHdl,iClear);
		DisableItem(editMenuHdl,iSelectAll);
	}		

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16  menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID	 = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
			{
				DoAboutBox();
			}
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			doFileMenu(menuItem);
			break;

		case mEdit:
			doEditMenu(menuItem);
			break;

		case mProgram:
			doErrorAlert(eProgramming);
/*#define mProgram				131
#define  iAssemble				1
#define  iAssembleAndProgram	2
#define  iProgramNoAssemble		3
#define  iVerifyFlash			4
#define  iLockChip				6
#define  iSetFuses				7 */
			break;

		case kHMHelpMenuID:
			if(FrontWindow())	
				doActivateDocWindow(FrontWindow(),false);
			doHelpMenu(menuItem);
			break;
	}

	HiliteMenu(0);
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;

	switch(menuItem)
	{
		case iNew:
			if(windowPtr = doNewDocWindow())
				ShowWindow(windowPtr);
			break;

		case iOpen:
			doOpenCommand();
			break;

		case iClose:
			doCloseWindow(FrontWindow());
			break;

		case iSaveAs:
			docStrucHdl = (docStructureHandle) (GetWRefCon(FrontWindow()));
			editRecHdl = (*docStrucHdl)->editRecHdl;
			doSaveAsFile(editRecHdl);
			break;

		case iQuit:
			gDone = true;
			break;
	}
}

//  doEditMenu

void	doEditMenu(SInt16 menuItem)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt32							totalSize, contigSize, newSize, scrapOffset;
	SInt16							selectionLength;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	switch(menuItem)
	{
		case iUndo:
			break;

		case iCut:
			if(ZeroScrap() == noErr)
			{
				PurgeSpace(&totalSize,&contigSize);
				selectionLength = doGetSelectLength(editRecHdl);
				if(selectionLength > contigSize)
					doErrorAlert(eNoSpaceCut);
				else
				{
					TECut(editRecHdl);
					doAdjustScrollbar(windowPtr);
					if(TEToScrap() != noErr)
						ZeroScrap();
				}
			}
			break;

		case iCopy:
			if(ZeroScrap() == noErr)
			{
				TECopy(editRecHdl);
				if(TEToScrap() != noErr)
					ZeroScrap();
			}
			break;

		case iPaste:
			newSize = (*editRecHdl)->teLength + GetScrap(NULL,'TEXT',&scrapOffset);
			if(newSize > kMaxTELength)
				doErrorAlert(eNoSpacePaste);
			else
			{
				if(TEFromScrap() == noErr)
				{
					TEPaste(editRecHdl);
					doAdjustScrollbar(windowPtr);
				}
			}
			break;

		case iClear:
			TEDelete(editRecHdl);
			doAdjustScrollbar(windowPtr);
			break;

		case iSelectAll:
			TESetSelect(0,(*editRecHdl)->teLength,editRecHdl);
			break;
	}

	doDrawDataPanel(windowPtr);
}

//  doGetSelectLength

SInt16  doGetSelectLength(TEHandle editRecHdl)
{
	SInt16	selectionLength;
		
	selectionLength = (*editRecHdl)->selEnd - (*editRecHdl)->selStart;
	return(selectionLength);
}

//  doAdjustScrollbar

void  doAdjustScrollbar(WindowPtr windowPtr)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							numberOfLines, controlMax, controlValue;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	numberOfLines = (*editRecHdl)->nLines;
	if(*(*(*editRecHdl)->hText + (*editRecHdl)->teLength - 1) == kReturn)
		numberOfLines += 1;

	controlMax = numberOfLines - (((*editRecHdl)->viewRect.bottom - 
							 (*editRecHdl)->viewRect.top) /
							 (*editRecHdl)->lineHeight);
	if(controlMax < 0)
		controlMax = 0;
	SetControlMaximum((*docStrucHdl)->vScrollbarHdl,controlMax);

	controlValue = ((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) / 
									(*editRecHdl)->lineHeight;
	if(controlValue < 0)
		controlValue = 0;
	else if(controlValue > controlMax)
		controlValue = controlMax;

	SetControlValue((*docStrucHdl)->vScrollbarHdl,controlValue);

#if TARGET_CPU_PPC
	if(gMacOS85Present)
	{
		SetControlViewSize((*docStrucHdl)->vScrollbarHdl,(*editRecHdl)->viewRect.bottom - 
											 (*editRecHdl)->viewRect.top);
	}
#endif
	
	TEScroll(0,((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) - 
					 		(GetControlValue((*docStrucHdl)->vScrollbarHdl) *
							(*editRecHdl)->lineHeight),editRecHdl);
}

//  doAdjustCursor

void  doAdjustCursor(WindowPtr windowPtr)
{
	GrafPtr		oldPort;
	RgnHandle	arrowRegion, iBeamRegion;
	Rect			cursorRect;
	Point			mouseXY;

	if(gInBackground)
	{
		SetCursor(&qd.arrow);
		return;
	}

	GetPort(&oldPort);
	SetPort(windowPtr);

	arrowRegion = NewRgn();
	iBeamRegion = NewRgn();	
	SetRectRgn(arrowRegion,-32768,-32768,32766,32766);

	cursorRect = windowPtr->portRect;
	cursorRect.bottom -= 15;
	cursorRect.right	-= 15;
	LocalToGlobal(&topLeft(cursorRect));
	LocalToGlobal(&botRight(cursorRect));	

	RectRgn(iBeamRegion,&cursorRect);
	DiffRgn(arrowRegion,iBeamRegion,arrowRegion);
	
	GetMouse(&mouseXY);
	LocalToGlobal(&mouseXY);

	if(PtInRgn(mouseXY,iBeamRegion))
	{
		SetCursor(*(GetCursor(iBeamCursor)));
		CopyRgn(iBeamRegion,gCursorRegion);
	}
	else
	{
		SetCursor(&qd.arrow);
		CopyRgn(arrowRegion,gCursorRegion);
	}

	DisposeRgn(arrowRegion);
	DisposeRgn(iBeamRegion);

	SetPort(oldPort);
}

//  doCloseWindow

void  doCloseWindow(WindowPtr windowPtr)
{
	docStructureHandle	docStrucHdl;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;

	DisposeControl((*docStrucHdl)->vScrollbarHdl);
	TEDispose((*docStrucHdl)->editRecHdl);
	DisposeHandle((Handle) docStrucHdl);
	DisposeWindow(windowPtr);

	gNumberOfWindows --;
}

//  doSaveAsFile

void  doSaveAsFile(TEHandle editRecHdl)
{
	StandardFileReply	fileReply;
	WindowPtr					windowPtr;
	SInt16						fileRefNum;
	SInt32						dataLength;
	Handle						editTextHdl;

	StandardPutFile("\pSave as:","\pUntitled",&fileReply);
	if(fileReply.sfGood)
	{
		windowPtr = FrontWindow();
		SetWTitle(windowPtr,fileReply.sfFile.name);

		if(!(fileReply.sfReplacing))
			FSpCreate(&fileReply.sfFile,' KJB','TEXT',fileReply.sfScript);

		FSpOpenDF(&fileReply.sfFile,fsCurPerm,&fileRefNum);

		dataLength = (*editRecHdl)->teLength;
		editTextHdl = (*editRecHdl)->hText;
		FSWrite(fileRefNum,&dataLength,*editTextHdl);

		FSClose(fileRefNum);
	}
}

//  doOpenCommand

void  doOpenCommand(void)
{
	StandardFileReply	fileReply;
	SFTypeList				fileTypes;	

	fileTypes[0] = 'TEXT';

	StandardGetFile(NULL,1,fileTypes,&fileReply);
	if(fileReply.sfGood)
		doOpenFile(fileReply.sfFile);
}

//  doOpenFile

void  doOpenFile(FSSpec fileSpec)
{
	WindowPtr 		windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle			editRecHdl;
	SInt16				fileRefNum;
	SInt32				textLength;
	Handle				textBuffer;

	if((windowPtr = doNewDocWindow()) == NULL)
		return;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	SetWTitle(windowPtr,fileSpec.name);

	FSpOpenDF(&fileSpec,fsCurPerm,&fileRefNum);

	SetFPos(fileRefNum,fsFromStart,0);
	GetEOF(fileRefNum,&textLength);

	if(textLength > 32767)
		textLength = 32767;

	textBuffer = NewHandle((Size) textLength);

	FSRead(fileRefNum,&textLength,*textBuffer);

	MoveHHi(textBuffer);
	HLock(textBuffer);

	TESetText(*textBuffer,textLength,editRecHdl);

	HUnlock(textBuffer);
	DisposeHandle(textBuffer);

	FSClose(fileRefNum);

	(*editRecHdl)->selStart = 0;
	(*editRecHdl)->selEnd = 0;

	ShowWindow(windowPtr);
}

//  doDrawDataPanel

void  doDrawDataPanel(WindowPtr windowPtr)
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	RGBColor						whiteColour = { 0xFFFF, 0xFFFF, 0xFFFF };					
	RGBColor						blackColour = { 0x0000, 0x0000, 0x0000 };					
	RGBColor						blueColour = { 0x1818, 0x4B4B, 0x8181 };
	ControlHandle					controlHdl;
	Rect							panelRect;
	Str255							textString;
	long							textpos1 = 127, textpos2 = 274, textpos3 = 400;
				
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;
	controlHdl = (*docStrucHdl)->vScrollbarHdl;

	MoveTo(0,282);
	LineTo(495,282);

	RGBForeColor(&whiteColour);
	RGBBackColor(&blueColour);
	SetRect(&panelRect,0,283,495,300);
	EraseRect(&panelRect);

	MoveTo(3,295);
	DrawString("\pNumber of characters =                   Number of Lines =                  Current Line =");

	SetRect(&panelRect,textpos1,284,textpos1 + 30,299);
	EraseRect(&panelRect);	
	SetRect(&panelRect,textpos2,284,textpos2 + 30,299);
	EraseRect(&panelRect);	
	SetRect(&panelRect,textpos3,284,textpos3 + 30,299);
	EraseRect(&panelRect);	

	NumToString((SInt32) (*editRecHdl)->teLength,textString);
	MoveTo(textpos1,295);
	DrawString(textString);

	NumToString((SInt32) (*editRecHdl)->nLines,textString);
	MoveTo(textpos2,295);
	DrawString(textString);

	NumToString((SInt32) currentLineNum(windowPtr),textString);
	MoveTo(textpos3,295);
	DrawString(textString);

	RGBForeColor(&blackColour);
	RGBBackColor(&whiteColour);
}

//  doErrorAlert

void  doErrorAlert(SInt16 errorCode)
{
	AlertStdAlertParamRec	paramRec;
	Str255								errorString;
	SInt16								itemHit;

	paramRec.movable				= true;
	paramRec.helpButton			= false;
	paramRec.filterProc			= NULL;
	paramRec.defaultText		= (StringPtr) kAlertDefaultOKText;
	paramRec.cancelText			= NULL;
	paramRec.otherText			= NULL;
	paramRec.defaultButton	= kAlertStdAlertOKButton;
	paramRec.cancelButton		= 0;
	paramRec.position				= kWindowDefaultPosition;

	GetIndString(errorString,rErrorStrings,errorCode);

	if(errorCode < eWindow)
	{
		StandardAlert(kAlertStopAlert,errorString,NULL,&paramRec,&itemHit);
		ExitToShell();
	}
	else
		StandardAlert(kAlertCautionAlert,errorString,NULL,&paramRec,&itemHit);
}

//  doHelpMenu

void	doHelpMenu(SInt16 menuItem)
{
	MenuHandle	helpMenuHdl;
	SInt16			origHelpItems, numItems;

	HMGetHelpMenuHandle(&helpMenuHdl);

	numItems = CountMenuItems(helpMenuHdl);
	origHelpItems = numItems - 1;

	if(menuItem > origHelpItems)
		doHelp();	
}

// 

long 	currentLineNum(WindowPtr windowPtr)
// Returns the line number of the currently selected line
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	long 							lineNum = 0;
				
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	while((*editRecHdl)->lineStarts[lineNum] < (*editRecHdl)->selStart)
	{
		lineNum++;
	}

	return lineNum;
}
// 

long 	currentCharacterPositionInLine(WindowPtr windowPtr)
// Returns the line number of the currently selected line
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	long 							lineNum, charNum;
				
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	lineNum = currentLineNum(windowPtr);
	
	charNum = (*editRecHdl)->selStart - (*editRecHdl)->lineStarts[lineNum - 1];

	return charNum;
}
// 

void DoAboutBox(void)
{
	DialogPtr			theDialog;
	
	theDialog = GetNewDialog(kAboutBoxDialogResID, nil, (WindowPtr) -1);
	SetPort(theDialog);
	DrawDialog(theDialog);
	
	while(!Button())
	{
		WaitNextEvent(everyEvent, NULL, 30, NULL);
	}
	
	DisposeDialog(theDialog);

	FlushEvents(everyEvent, 0);
}
// 

