#include <DriverServices.h>
#include <TextUtils.h>
#include <ControlDefinitions.h>
#include <Controls.h>
#include "TextWindowSupport.h"
#include "MacAVRpa.h"
#include "MenuSupport.h"

//  global variables

ControlActionUPP			scrollActionFunctionUPP;
TEClickLoopUPP				customClickLoopUPP;


//  doSetupTextUPPs
void doSetupTextUPPs(void)
{
	scrollActionFunctionUPP = NewControlActionProc((ProcPtr) scrollActionFunction);
	customClickLoopUPP = NewTEClickLoopProc((ProcPtr) customClickLoop);
}



//  doKeyEvent

void  doKeyEvent(SInt8 charCode)
{
	WindowPtr						windowPtr;
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	AVRFileInfoHandle				AVRFileInfoHdl;
	SInt16							selectionLength;
	short 							charOffset;
	short 							lineNum;
	Handle							theTextHandle;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;
	AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;

	if(charCode == kTab)
	{
		// Insert a space character...
		TEKey(' ',editRecHdl);
		
		// then keep inserting spaces until the cursor position is a multiple of 3.
		while((currentCharacterPositionInLine(windowPtr) % 3) != 0)
		{
			TEKey(' ',editRecHdl);
		}
		
		doAdjustScrollbar(windowPtr);
		
	}
	else if(charCode == kReturn)
	// This function will indent the next line with the previous one, ala Codewarrior
	{
		lineNum = currentLineNum(windowPtr);
		theTextHandle = (*editRecHdl)->hText;
		
		// Insert a return character...
		TEKey(kReturn,editRecHdl);
		
		charOffset = currentCharacterPositionInLine(windowPtr); 

		// then keep inserting spaces while the previous line has them.
		while((*theTextHandle)[(*editRecHdl)->selStart - charOffset] == ' ')
		{
			TEKey(' ',editRecHdl);
		}
		
		doAdjustScrollbar(windowPtr);
	}
	else if(charCode == kDel)
	{
		selectionLength = doGetSelectLength(editRecHdl);
		if(selectionLength == 0)
			(*editRecHdl)->selEnd += 1;
		TEDelete(editRecHdl);
		doAdjustScrollbar(windowPtr);
	}
	else
	{
		selectionLength = doGetSelectLength(editRecHdl);
		if(((*editRecHdl)->teLength - selectionLength + 1) < kMaxTELength)
		{
			TEKey(charCode,editRecHdl);
			doAdjustScrollbar(windowPtr);
		}
		else
			doErrorAlert(eExceedChara);
	}

	(*AVRFileInfoHdl)->dirty = true;

	doDrawDataPanel(windowPtr);
}

//  scrollActionFunction

pascal void  scrollActionFunction(ControlHandle controlHdl,SInt16 partCode)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							linesToScroll;
	SInt16							controlValue, controlMax;

	windowPtr = (*controlHdl)->contrlOwner;
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	controlValue = GetControlValue(controlHdl);
	controlMax = GetControlMaximum(controlHdl);

	if(partCode)
	{
		if(partCode != kControlIndicatorPart)
		{
			switch(partCode)
			{
				case kControlUpButtonPart:
				case kControlDownButtonPart:
					linesToScroll = 1;
					break;
				
				case kControlPageUpPart:
				case kControlPageDownPart:
					linesToScroll = (((*editRecHdl)->viewRect.bottom - (*editRecHdl)->viewRect.top) /
													 (*editRecHdl)->lineHeight) - 1;
					break;
			}

			if((partCode == kControlDownButtonPart) || (partCode == kControlPageDownPart))
				linesToScroll = -linesToScroll;

			linesToScroll = controlValue - linesToScroll;
			if(linesToScroll < 0)
				linesToScroll = 0;
			else if(linesToScroll > controlMax)
				linesToScroll = controlMax;

			SetControlValue(controlHdl,linesToScroll);

			linesToScroll = controlValue - linesToScroll;
		}
		else
		{
			linesToScroll = gOldControlValue - controlValue;
			gOldControlValue = controlValue;
		}

		if(linesToScroll != 0)
		{
			TEScroll(0,linesToScroll * (*editRecHdl)->lineHeight,editRecHdl);
			doDrawDataPanel(windowPtr);
		}
	}
}

//  doInContent

void  doInContent(EventRecord *eventStrucPtr)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	Point								mouseXY;
	ControlHandle				controlHdl;
	SInt16							partCode;
	Boolean							shiftKeyPosition = false;

	windowPtr = FrontWindow();
	
	if(IsDialogEvent(eventStrucPtr))
	{
	
	} else
	{
		docStrucHdl = (docStructureHandle) GetWRefCon(windowPtr);
		editRecHdl = (*docStrucHdl)->editRecHdl;

		mouseXY = eventStrucPtr->where;
		SetPort(windowPtr);
		GlobalToLocal(&mouseXY);

		if((partCode = FindControl(mouseXY,windowPtr,&controlHdl)) != 0)
		{
			gOldControlValue = GetControlValue(controlHdl);
			TrackControl(controlHdl,mouseXY,scrollActionFunctionUPP);
		}
		else if(PtInRect(mouseXY,&(*editRecHdl)->viewRect))
		{
			if((eventStrucPtr->modifiers & shiftKey) != 0)
				shiftKeyPosition = true;
			TEClick(mouseXY,shiftKeyPosition,editRecHdl);
		}
		
		doDrawDataPanel(windowPtr);
	}
}

//  doUpdateDocument

void  doUpdateDocument(WindowPtr windowPtr)
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	GrafPtr							oldPort;
	Rect 								contentRect;
	OSStatus						osError;
	UInt32							actualSize;
	

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	
	GetPort(&oldPort);
	SetPort(windowPtr);

	BeginUpdate(windowPtr);


	EraseRgn(windowPtr->visRgn);
	DrawGrowIcon(windowPtr);

	contentRect = windowPtr->portRect;
	contentRect.right -= 17;
	contentRect.bottom -= 18;
	editRecHdl = (*docStrucHdl)->editRecHdl;
	(*editRecHdl)->destRect = (*editRecHdl)->viewRect = contentRect;

	(*editRecHdl)->viewRect.bottom = ((((*editRecHdl)->viewRect.bottom -
																			(*editRecHdl)->viewRect.top) / 
																			(*editRecHdl)->lineHeight) * 
																			(*editRecHdl)->lineHeight) + 
																			(*editRecHdl)->viewRect.top;
	(*editRecHdl)->destRect.bottom = (*editRecHdl)->viewRect.bottom;
	
	(*(*docStrucHdl)->vScrollbarHdl)->contrlRect.left = windowPtr->portRect.right - 16;		
	(*(*docStrucHdl)->vScrollbarHdl)->contrlRect.right = windowPtr->portRect.right;
	(*(*docStrucHdl)->vScrollbarHdl)->contrlRect.top = windowPtr->portRect.top;
	(*(*docStrucHdl)->vScrollbarHdl)->contrlRect.bottom = windowPtr->portRect.bottom - 17;
//		TECalText(editRecHdl);
	TECalText((*docStrucHdl)->editRecHdl);

	TEScroll(0,((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) - 
				 		(GetControlValue((*docStrucHdl)->vScrollbarHdl) *
						(*editRecHdl)->lineHeight),editRecHdl);

	TEUpdate(&contentRect,(*docStrucHdl)->editRecHdl);


	UpdateControls(windowPtr,windowPtr->visRgn);


	doDrawDataPanel(windowPtr);
	doAdjustScrollbar(windowPtr);


	EndUpdate(windowPtr);

	SetPort(oldPort);
	
}

//  doActivateDocWindow

void  doActivateDocWindow(WindowPtr windowPtr,Boolean becomingActive)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;


	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	if(becomingActive)
	{
		SetPort(windowPtr);

		(*editRecHdl)->viewRect.bottom = ((((*editRecHdl)->viewRect.bottom -
																				(*editRecHdl)->viewRect.top) / 
																				(*editRecHdl)->lineHeight) * 
																				(*editRecHdl)->lineHeight) + 
																				(*editRecHdl)->viewRect.top;
		(*editRecHdl)->destRect.bottom = (*editRecHdl)->viewRect.bottom;

		TEActivate(editRecHdl);
//		ActivateControl((*docStrucHdl)->vScrollbarHdl);      - Crashes with Unimplemented Trap in System 7.1
		HiliteControl((*docStrucHdl)->vScrollbarHdl,0);		// System 7.1 friendly version 
		doAdjustScrollbar(windowPtr);
	}
	else
	{
		TEDeactivate(editRecHdl);
//		DeactivateControl((*docStrucHdl)->vScrollbarHdl); 	- Crashes with Unimplemented Trap in System 7.1
		HiliteControl((*docStrucHdl)->vScrollbarHdl,255);		// System 7.1 friendly version 
	}
}

//  doNewDocWindow

WindowPtr  doNewDocWindow(void)
{
	WindowPtr 					windowPtr;
	docStructureHandle			docStrucHdl;
	Rect						destAndViewRect;
	TEHandle					editRecHdl;
	AVRFileInfoHandle			AVRFileInfoHdl;

	if(!(windowPtr = GetNewCWindow(rWindow,NULL,(WindowPtr) -1)))
	{
		doErrorAlert(eWindow);
		return(NULL);
	}

	SetPort(windowPtr);
	TextSize(10);
	TextFont(kFontIDCourier);

	if(!(docStrucHdl = (docStructureHandle) NewHandle(sizeof(docStructure))))
	{
		doErrorAlert(eDocStructure);
		return(NULL);
	}
	SetWRefCon(windowPtr,(SInt32) docStrucHdl);

	gNumberOfWindows ++;

	(*docStrucHdl)->vScrollbarHdl = GetNewControl(rVScrollbar,windowPtr);
	
	destAndViewRect = windowPtr->portRect;
	destAndViewRect.right -= 15;
	destAndViewRect.bottom -= 15;
	InsetRect(&destAndViewRect,2,2);

	MoveHHi((Handle) docStrucHdl);
	HLock((Handle) docStrucHdl);

	if(!((*docStrucHdl)->editRecHdl = TENew(&destAndViewRect,&destAndViewRect)))
	{
		DisposeWindow(windowPtr);
		gNumberOfWindows --;
		DisposeHandle((Handle) docStrucHdl);
		doErrorAlert(eEditRecord);
		return(NULL);
	}
	
	HUnlock((Handle) docStrucHdl);
	
	TESetClickLoop(customClickLoopUPP,(*docStrucHdl)->editRecHdl);
	TEAutoView(true,(*docStrucHdl)->editRecHdl);
	TEFeatureFlag(teFOutlineHilite,1,(*docStrucHdl)->editRecHdl);


// Modified for courier


	editRecHdl = (*docStrucHdl)->editRecHdl;
	(*editRecHdl)->txFont = kFontIDCourier;
	
// Set up the file info for that window...
	(*docStrucHdl)->AVRFileInfoHdl = new (AVRFileInfo*);
	AVRFileInfoHdl = (*docStrucHdl)->AVRFileInfoHdl;
	(*AVRFileInfoHdl) = new AVRFileInfo;
	
	(*AVRFileInfoHdl)->hasBeenSaved = false;
	(*AVRFileInfoHdl)->dirty = true;
	PStrCopy((*AVRFileInfoHdl)->AVRFSSpec.name, "\pUntitled.asm");
	return(windowPtr);
}

//  customClickLoop

pascal Boolean  customClickLoop(void)
{
	WindowPtr						windowPtr;
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	GrafPtr							oldPort;
	RgnHandle						oldClip;
	Rect								tempRect;
	Point								mouseXY;
	SInt16							linesToScroll = 0;

	windowPtr = FrontWindow();
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	GetPort(&oldPort);
	SetPort(windowPtr);
	oldClip = NewRgn();
	GetClip(oldClip);
	SetRect(&tempRect,-32767,-32767,32767,32767);
	ClipRect(&tempRect);

	GetMouse(&mouseXY);
	if(mouseXY.v < windowPtr->portRect.top)
	{
		linesToScroll = 1;
		doSetScrollBarValue((*docStrucHdl)->vScrollbarHdl,&linesToScroll);
		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * ((*editRecHdl)->lineHeight),editRecHdl);
	}
	else if(mouseXY.v > windowPtr->portRect.bottom)
	{
		linesToScroll = -1;
		doSetScrollBarValue((*docStrucHdl)->vScrollbarHdl,&linesToScroll);
		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * ((*editRecHdl)->lineHeight),editRecHdl);
	}

	if(linesToScroll != 0)
		doDrawDataPanel(windowPtr);

	SetClip(oldClip);
	DisposeRgn(oldClip);
	SetPort(oldPort);

	return(true);
}

//  doSetScrollBarValue

void  doSetScrollBarValue(ControlHandle controlHdl,SInt16 *linesToScroll)
{
	SInt16	controlValue, controlMax;

	controlValue = GetControlValue(controlHdl);
	controlMax = GetControlMaximum(controlHdl);

	*linesToScroll = controlValue - *linesToScroll;
	if(*linesToScroll < 0)
		*linesToScroll = 0;
	else if(*linesToScroll > controlMax)
		*linesToScroll = controlMax;

	SetControlValue(controlHdl,*linesToScroll);
	*linesToScroll = controlValue - *linesToScroll;
}

//  doGetSelectLength

SInt16  doGetSelectLength(TEHandle editRecHdl)
{
	SInt16	selectionLength;
		
	selectionLength = (*editRecHdl)->selEnd - (*editRecHdl)->selStart;
	return(selectionLength);
}

//  doAdjustScrollbar

void  doAdjustScrollbar(WindowPtr windowPtr)
{
	docStructureHandle	docStrucHdl;
	TEHandle						editRecHdl;
	SInt16							numberOfLines, controlMax, controlValue;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	numberOfLines = (*editRecHdl)->nLines;
	if(*(*(*editRecHdl)->hText + (*editRecHdl)->teLength - 1) == kReturn)
		numberOfLines += 1;

	controlMax = numberOfLines - (((*editRecHdl)->viewRect.bottom - 
							 (*editRecHdl)->viewRect.top) /
							 (*editRecHdl)->lineHeight);
	if(controlMax < 0)
		controlMax = 0;
	SetControlMaximum((*docStrucHdl)->vScrollbarHdl,controlMax);

	controlValue = ((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) / 
									(*editRecHdl)->lineHeight;
	if(controlValue < 0)
		controlValue = 0;
	else if(controlValue > controlMax)
		controlValue = controlMax;

	SetControlValue((*docStrucHdl)->vScrollbarHdl,controlValue);

#if TARGET_CPU_PPC && !defined MACOS_800
	if(gMacOS85Present)
	{
		SetControlViewSize((*docStrucHdl)->vScrollbarHdl,((*editRecHdl)->viewRect.bottom - 
											 (*editRecHdl)->viewRect.top) / (*editRecHdl)->lineHeight);
	}
#endif
	
	TEScroll(0,((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) - 
					 		(GetControlValue((*docStrucHdl)->vScrollbarHdl) *
							(*editRecHdl)->lineHeight),editRecHdl);
}

//  doAdjustCursor

void  doAdjustCursor(WindowPtr windowPtr)
{
	GrafPtr		oldPort;
	RgnHandle	arrowRegion, iBeamRegion;
	Rect			cursorRect;
	Point			mouseXY;

	if(gInBackground)
	{
		SetCursor(&qd.arrow);
		return;
	}

	GetPort(&oldPort);
	SetPort(windowPtr);

	arrowRegion = NewRgn();
	iBeamRegion = NewRgn();	
	SetRectRgn(arrowRegion,-32768,-32768,32766,32766);

	cursorRect = windowPtr->portRect;
	cursorRect.bottom -= 15;
	cursorRect.right	-= 15;
	LocalToGlobal(&topLeft(cursorRect));
	LocalToGlobal(&botRight(cursorRect));	

	RectRgn(iBeamRegion,&cursorRect);
	DiffRgn(arrowRegion,iBeamRegion,arrowRegion);
	
	GetMouse(&mouseXY);
	LocalToGlobal(&mouseXY);

	if(PtInRgn(mouseXY,iBeamRegion))
	{
		SetCursor(*(GetCursor(iBeamCursor)));
		CopyRgn(iBeamRegion,gCursorRegion);
	}
	else
	{
		SetCursor(&qd.arrow);
		CopyRgn(arrowRegion,gCursorRegion);
	}

	DisposeRgn(arrowRegion);
	DisposeRgn(iBeamRegion);

	SetPort(oldPort);
}

//  doCloseWindow

void  doCloseWindow(WindowPtr windowPtr)
{
	docStructureHandle	docStrucHdl;

	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;

	DisposeControl((*docStrucHdl)->vScrollbarHdl);
	TEDispose((*docStrucHdl)->editRecHdl);
	delete (*((*docStrucHdl)->AVRFileInfoHdl));
	DisposeHandle((Handle) docStrucHdl);
	DisposeWindow(windowPtr);

	gNumberOfWindows --;
	doAdjustMenus();
}

//  doDrawDataPanel

void  doDrawDataPanel(WindowPtr windowPtr)
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
// Start dodgy 68k kludges

	RGBColor						whiteColour = { 0xFFFF, 0xFFFF, 0xFFFF };					
	RGBColor						blackColour = { 0x0000, 0x0000, 0x0000 };					
	RGBColor						blueColour = { 0x1818, 0x4B4B, 0x8181 };
	ControlHandle					controlHdl;
	Rect							panelRect;
	Str255							textString;
	long							textpos1 = 127, textpos2 = 300, textpos3 = 435;
		
	if(windowPtr)
	{		
		docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
		editRecHdl = (*docStrucHdl)->editRecHdl;
		controlHdl = (*docStrucHdl)->vScrollbarHdl;

		RGBForeColor(&whiteColour);
		RGBBackColor(&blueColour);
		SetRect(&panelRect,windowPtr->portRect.left,windowPtr->portRect.bottom - 18,windowPtr->portRect.right,windowPtr->portRect.bottom);
		EraseRect(&panelRect);

		MoveTo(3,windowPtr->portRect.bottom - 5);
		DrawString("\pNumber of characters =         Number of Lines =        Current Line =");

		SetRect(&panelRect,textpos1,windowPtr->portRect.bottom - 17,textpos1 + 30,windowPtr->portRect.bottom);
		EraseRect(&panelRect);	
		SetRect(&panelRect,textpos2,windowPtr->portRect.bottom - 17,textpos2 + 30,windowPtr->portRect.bottom);
		EraseRect(&panelRect);	
		SetRect(&panelRect,textpos3,windowPtr->portRect.bottom,textpos3 + 30,windowPtr->portRect.bottom);
		EraseRect(&panelRect);	

		NumToString((SInt32) (*editRecHdl)->teLength,textString);
		MoveTo(textpos1,windowPtr->portRect.bottom - 5);
		DrawString(textString);

		NumToString((SInt32) (*editRecHdl)->nLines,textString);
		MoveTo(textpos2,windowPtr->portRect.bottom - 5);
		DrawString(textString);

		NumToString((SInt32) currentLineNum(windowPtr),textString);
		MoveTo(textpos3,windowPtr->portRect.bottom - 5);
		DrawString(textString);

		RGBForeColor(&blackColour);
		RGBBackColor(&whiteColour);
	}
}

// 

long 	currentLineNum(WindowPtr windowPtr)
// Returns the line number of the currently selected line
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	long 							lineNum = 0;
				
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	while((*editRecHdl)->lineStarts[lineNum] < (*editRecHdl)->selStart)
	{
		lineNum++;
	}

	return lineNum;
}
// 

long 	currentCharacterPositionInLine(WindowPtr windowPtr)
// Returns the line number of the currently selected line
{
	docStructureHandle				docStrucHdl;
	TEHandle						editRecHdl;
	long 							lineNum, charNum;
				
	docStrucHdl = (docStructureHandle) (GetWRefCon(windowPtr));;
	editRecHdl = (*docStrucHdl)->editRecHdl;

	lineNum = currentLineNum(windowPtr);
	
	charNum = (*editRecHdl)->selStart - (*editRecHdl)->lineStarts[lineNum - 1];

	return charNum;
}

