/*  Metrowerks Standard Library  */

/*  $Date: 1999/01/22 23:42:11 $ 
 *  $Revision: 1.2 $ 
 *  $NoKeywords: $ 
 *
 *		Copyright  1995-1999 Metrowerks, Inc.
 *		All rights reserved.
 */

/*
 *	File:		unistd.h
 *
 *	Content:	Interface file to standard UNIX-style entry points ...
 *
 *	NB:			This file implements some UNIX low level support.  These functions
 *				are not guaranteed to be 100% conformant.
 */

#ifndef	_UNISTD
#define	_UNISTD

#include <ansi_parms.h>                 /* mm 970905*/

#pragma options align=native
#if defined(__CFM68K__) && !defined(__USING_STATIC_LIBS__)
	#pragma import on
#endif
/* #pragma direct_destruction off */  /* vss 980807 */

#ifndef __stdio__
	/* macros for whence parameter of lseek() (taken from <stdio.h> */
	#define SEEK_SET	0
	#define SEEK_CUR	1
	#define SEEK_END	2
#endif

#ifdef __cplusplus
	extern "C" {
#endif

/*
 *	Change the current directory.
 */
int chdir(const char *path);

/*
 *	Closes an open file.
 */
int close(int fildes);

/*
 *	Returns the user name associated with the current process.  For the mac we always return
 *	the login name.  If string is not NULL, it must be at least FILENAME_MAX large.
 */
char *cuserid(char *string);

/*
 *	Launches the application fname and then quits upon succesful launch.
 *	NB: all exec calls pass through this one call, since argument passing (argc, argv) doesn't
 *		exist for mac applications.
 */
int exec(const char *path, ...);
#define execl			exec
#define execv			exec
#define execle			exec
#define execle			exec
#define execve			exec
#define execlp			exec
#define execvp			exec

/*
 *	Get the current directory.
 */
char *getcwd(char *buf, int size);

/*
 *	The following UNIX functions don't really have any meaning on the Mac, so we just
 *	return values that would make sense for a typical user process under UNIX ...
 */

#define getpid()		((int) 9000)
#define getppid()		((int) 8000)
#define getuid()		((int) 200)
#define geteuid()		((int) 200)
#define getgid()		((int) 100)
#define getegid()		((int) 100)
#define getpgrp()		((int) 9000)

/*
 *	The Mac doesn't have a login, so we just return the Owner Name from the Sharing Setup
 *	Control Panel.
 */
char *getlogin(void);

/*
 *	Determines is a specified fileid is attached to the console.
 */
int isatty(int fildes);

/*
 *	Seek on a file stream.
 */
long lseek(int fildes, long offset, int whence);

/*
 *	Reads from a file stream.
 */
int read(int fildes, char *buf, int count);

/*
 *	Delete a directory.
 */
int rmdir(const char *path);

/*
 *	Delay program execution for a specified number of seconds.
 */
unsigned int sleep(unsigned int sleep);

/*
 *	Returns the name of the terminal associated with the fileid, or NULL if fileid doesn't
 *	specify a terminal.
 */
char *ttyname(int fildes);

/*
 *	Unlink (delete) a file.
 */
int unlink(const char *path);

/*
 *	Writes to a file stream.
 */
int write(int fildes, const char *buf, int count);

#ifdef __cplusplus
	}
#endif

/* #pragma direct_destruction reset */  /* 980807 vss */
#if defined(__CFM68K__) && !defined(__USING_STATIC_LIBS__)
	#pragma import reset
#endif
#pragma options align=reset

#endif
/*     Change record
 mm 970905  added include of ansi_parms.h to avoid need for prefix file
 vss 980807 remove pragma  - no longer supported by compiler
*/

void catdirname(char *buf, int size, short vrefnum, long dirnum);
