/***********************************************************************
 *  args - Argument reading system
 *  Copyright (C) 1997-1999 Jon Anders Haugum
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to
 *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *
 *
 *  Author of args can be reached at:
 *     email: jonah@omegav.ntnu.no
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "macAVRpa.h"
#include "misc.h"
#include "args.h"


struct args *alloc_args(int arg_count)
	{
	struct args *argsPtr = NULL;

//	args = malloc(sizeof(struct args));
	argsPtr = new(args);
	if(argsPtr != NULL)
		{
//		argsPtr->arg = malloc(sizeof(struct arg) * arg_count);
		argsPtr->argPtr = new arg[arg_count];
		if(argsPtr->argPtr != NULL)
			{
			argsPtr->count = arg_count;
			argsPtr->first_data = NULL;
			return(argsPtr);
			}
		delete argsPtr;
		}
	doCautionAlert("\pError: Unable to allocate memory!");
	return(NULL);
	}



int read_args(struct args *argsPtr, int argc, char *argv[])
	{
	int i, j, k, ok = True, i_old;
	struct data_list **last_data;

	argsPtr->first_data = NULL;
	last_data = &argsPtr->first_data;
	for(i = 1; (i < argc) && ok; i++)
		{
		if(argv[i][0] == '-')
			{
			last_data = &argsPtr->first_data;
			if(argv[i][1] == '-')
				{
				j = 0;
				while((argsPtr->argPtr[j].longarg ? strcmp(&argv[i][2], argsPtr->argPtr[j].longarg) : True) && (j != argsPtr->count))
					j++;
				if(j == argsPtr->count)
					{
//					doCautionAlert("\pError: Unknown option: %s\n", argv[i]);
					char errorAlert[200];
					strcpy(errorAlert, "Error: Unknown option: ");
					strcat(errorAlert, argv[1]);
					C2PStr(errorAlert);
					doCautionAlert((unsigned char*)errorAlert);
					ok = False;
					}
				else
					{
					switch(argsPtr->argPtr[j].type)
						{
						case ARGTYPE_STRING:
						case ARGTYPE_STRING_MULTISINGLE:
							if((i + 1) == argc)
								{
								doCautionAlert("\pError: No argument supplied with option.");
								ok = False;
								}
							else if(argsPtr->argPtr[j].type != ARGTYPE_STRING_MULTISINGLE)
								argsPtr->argPtr[j].data = argv[++i];
							else
								ok = add_arg((struct data_list **)&argsPtr->argPtr[j].data, argv[++i]);
							break;
						case ARGTYPE_BOOLEAN:
							argsPtr->argPtr[j].data = (char *)True;
							break;
						case ARGTYPE_STRING_MULTI:
							last_data = (struct data_list **)&argsPtr->argPtr[j].data;
							break;
						}
					}
				}
			else
				{
				// ToDo: Sjekk for enkel -
				for(k = 1, i_old = i; (argv[i][k] != '\0') && ok && (i == i_old); k++)
					{
					j = 0;
					while((argv[i][k] != argsPtr->argPtr[j].letter) && (j != argsPtr->count))
						j++;
					if(j == argsPtr->count)
						{
						doCautionAlert("\pError: Unknown option: -%c\n");
						ok = False;
						}
					else
						{
						switch(argsPtr->argPtr[j].type)
							{
							case ARGTYPE_STRING:
								if(argv[i][k + 1] != '\0')
									{
									doCautionAlert("\pError: Option -%c must be followed by it's argument");
									ok = False;
									}
								else
									{
									if((i + 1) == argc)
										{
//										doCautionAlert("\pError: No argument supplied with option: -%c\n", argv[i][k]);
										doCautionAlert("\pError: No argument supplied with option: -%c");
										ok = False;
										}
									else if(argsPtr->argPtr[j].type != ARGTYPE_STRING_MULTISINGLE)
										argsPtr->argPtr[j].data = argv[++i];
									else
										ok = add_arg((struct data_list **)&argsPtr->argPtr[j].data, argv[++i]);
									}
								break;
							case ARGTYPE_BOOLEAN:
								argsPtr->argPtr[j].data = (char *)True;
								break;
							case ARGTYPE_STRING_MULTI:
								last_data = (struct data_list **)&argsPtr->argPtr[j].data;
								break;
							}
						}
					}
				}
			}
		else
			ok = add_arg(last_data, argv[i]);
		}
	return(ok);
	}


int add_arg(struct data_list **last_data, char *argv)
	{
	struct data_list *data;

	while(*last_data) last_data = &((*last_data)->next);
	data = new data_list;
	if(data)
		{
		data->next = NULL;
		data->data = argv;
		*last_data = data;
		last_data = &data->next;
		}
	else
		{
		doCautionAlert("\pError: Unable to allocate memory!");
		return(False);
		}
	return(True);
	}


void free_args(struct args *argsPtr)
	{
	int i;
	struct data_list *data, *temp;

	for(data = argsPtr->first_data; data;)
		{
		temp = data;
		data = data->next;
		free(temp);
		}
	for(i = 0; i != argsPtr->count; i++)
		if((argsPtr->argPtr[i].type == ARGTYPE_STRING_MULTI)
		   || (argsPtr->argPtr[i].type == ARGTYPE_STRING_MULTISINGLE))
			for(data = (data_list*)argsPtr->argPtr[i].data; data;)
				{
				temp = data;
				data = data->next;
				free(temp);
				}
	delete argsPtr;
	}


void define_arg(struct args *argsPtr, int index, int type, char letter, char *longarg, void *def_value)
	{
	argsPtr->argPtr[index].type = type;
	argsPtr->argPtr[index].letter = letter;
	argsPtr->argPtr[index].longarg = longarg;
	argsPtr->argPtr[index].data = def_value;
	}
	
void * GET_ARG(args* theargs, int argnum)
{
	return (theargs->argPtr[argnum].data);
}

void SET_ARG(args* theargs,int argnum,void* value)
{
	theargs->argPtr[argnum].data = (void *)value;
}


