/***********************************************************************
 *  avra - Assembler for the Atmel AVR microcontroller series
 *  Copyright (C) 1998-2000 Jon Anders Haugum
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to
 *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *
 *
 *  Author of avra can be reached at:
 *     email: jonah@omegav.ntnu.no
 *     www: http://www.omegav.ntnu.no/~jonah/el/avra.html
 *
 *
 *  AVRA has been havily hacked, maimed, and generally ripped apart in 
 *  an attempt to coerce it to work under the MacOS.  Any memory leaks
 *  or bugs are probably the fault of Ben Hitchcock.
 *
 *  You can tell Ben about his lack of programming prowess by emailing
 *  ben@wollongong.apana.org.au.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "macavrpa.h"
#include "misc.h"
#include "args.h"
#include "avra.h"
#include "device.h"


const char *title = "avra %d.%d (%s)\n"
                    "Copyright (C) 1998-2000 Jon Anders Haugum\n"
                    "   avra comes with NO WARRANTY, to the extent permitted by law.\n"
                    "   You may redistribute copies of avra under the terms\n"
                    "   of the GNU General Public License.\n"
                    "   For more information about these matters, see the files named COPYING.\n";

const char *usage =
	"usage: avra [--define <symbol>[=<value>]] [--listmac]\n"
	"            [--max_errors <number>] [--version] [-h] [--help]\n"
	"            <file to assemble>\n"
	"\n"
	"   --define     : Define symbol.\n"
	"   --listmac    : List macro expansion in listfile\n"
	"   --max_errors : Maximum number of errors before exit (default: 10)\n"
	"   --version    : Version information\n"
	"   --help, -h   : This help text\n"
	"\n"
	"Report bugs to jonah@omegav.ntnu.no\n";



int avramain(int argc, char *argv[])
	{
	int show_usage = False;
	struct prog_info *pi = NULL;
	struct args *argsPtr = NULL;

	argsPtr = alloc_args(ARG_COUNT);
	if(argsPtr)
		{
		define_arg(argsPtr, ARG_DEFINE,     ARGTYPE_STRING_MULTISINGLE,   0, "define",     NULL);
		define_arg(argsPtr, ARG_LISTMAC,    ARGTYPE_BOOLEAN,              0, "listmac",    NULL);
		define_arg(argsPtr, ARG_MAX_ERRORS, ARGTYPE_STRING,               0, "max_errors", NULL);
		define_arg(argsPtr, ARG_VER,        ARGTYPE_BOOLEAN,              0, "version",    NULL);
		define_arg(argsPtr, ARG_HELP,       ARGTYPE_BOOLEAN,            'h', "help",       NULL);
		if(read_args(argsPtr, argc, argv))
			{
			if(!GET_ARG(argsPtr, ARG_HELP) && (argc != 1))
				{
				if(!GET_ARG(argsPtr, ARG_VER))
					{
					pi = get_pi(argsPtr);
					if(pi)
						{
						assemble(pi);
						free_pi(pi);
						}
					}
				else
					doCautionAlert("\ptitle");
					//, VERSION, REVISION, DATESTRING);
				}
			else
				show_usage = True;
			}
		free_args(argsPtr);
		}
	else
		{
		show_usage = True;
//		printf("\n");
		}
	if(show_usage)
		{
		if(argc == 1)
		        {
			doCautionAlert("\ptitle");
			//, VERSION, REVISION, DATESTRING);
			//printf("\n");
			}
		//printf("%s", usage);
		}
	return(EXIT_SUCCESS);
	}


void assemble(struct prog_info *pi)
        {

	if(pi->argsPtr->first_data)
	        {
		setStatusText("Pass 1...");

		if(!load_arg_defines(pi))
		        return;
		if(parse_file(pi, (char *)pi->argsPtr->first_data->data, PASS_1))
		        {
			if(pi->error_count == 0)
			        {
				prepare_second_pass(pi);
				if(!load_arg_defines(pi))
				        return;
				if(open_out_files(pi, (char*) pi->argsPtr->first_data->data))
				        {
					setStatusText("Pass 2...");
					parse_file(pi, (char *)pi->argsPtr->first_data->data, PASS_2);
					close_out_files(pi);
					write_map_file(pi);
					if(pi->error_count)
						{
							char errorString[100];
							unsigned char numString[10];
							strcpy(errorString, "Assembly aborted with ");
							NumToString((SInt32) pi->error_count,(unsigned char*)numString);
							P2CStrCat(errorString, numString);
							strcat(errorString, " errors ");
							NumToString((SInt32) pi->warning_count,(unsigned char*)numString);
							P2CStrCat(errorString, numString);
							strcat(errorString, " warnings.");
							C2PStr(errorString);
					        doCautionAlert((unsigned char*)errorString);
					    }
					        //, pi->error_count, pi->warning_count);
					else if(pi->warning_count)
						{
							char errorText[500];
							char errorNum[20];
							strcpy(errorText, "Assembly complete with no errors (");
							strcpy(errorText, errorNum);
							strcpy(errorText, " warnings)");
							C2PStr(errorText);
					        doCautionAlert((unsigned char*)errorText);
					    }
					        //, pi->warning_count);
					else
					        doNoteAlert("\pAssembly complete with no errors.");
					}
				}
			}
		}
	else
	        doCautionAlert("\pError: You need to specify a file to assemble\n");
	}


int load_arg_defines(struct prog_info *pi)
        {
	int i;
	char *expr, buff[256];
	struct data_list *define;
	struct label *labelPtr;

	for(define = (data_list*)GET_ARG(pi->argsPtr, ARG_DEFINE); define; define = define->next)
	        {
		strcpy(buff, (char*)define->data);
		expr = get_next_token(buff, TERM_EQUAL);
		if(expr)
		        {
			if(!get_expr(pi, expr, &i))
			        return(False);
			}
		else
		        i = 1;
		for(labelPtr = pi->first_constant; labelPtr; labelPtr = labelPtr->next)
		        if(!nocase_strcmp(labelPtr->name, buff))
			        {
				doCautionAlert("\pError: Can't define symbol twice!");
				//, buff);
				return(False);
				}
//		label = malloc(sizeof(struct label));
		labelPtr = new label;
		if(!labelPtr)
		        {
			print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
			return(False);
			}
		labelPtr->next = NULL;
		if(pi->last_constant)
		        pi->last_constant->next = labelPtr;
		else
		        pi->first_constant = labelPtr;
		pi->last_constant = labelPtr;
//		labelPtr->name = malloc(strlen(buff) + 1);
		labelPtr->name = new char[strlen(buff) + 1];
		if(!labelPtr->name)
		        {
			print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
			return(False);
			}
		strcpy(labelPtr->name, buff);
		labelPtr->value = i;
		}
	return(True);
	}


struct prog_info *get_pi(struct args *argsPtr)
	{
	struct prog_info *pi;
	long	maxerrors = 0;

	pi = (struct prog_info *)calloc(1, sizeof(struct prog_info));
	if(pi)
		{
		pi->argsPtr = argsPtr;
		pi->devicePtr = get_device(NULL);
		pi->list_on = True;
		pi->segment = SEGMENT_CODE;
		pi->dseg_addr = DSEG_START;
		maxerrors = atoi((char*)GET_ARG(argsPtr, ARG_MAX_ERRORS));
		pi->max_errors = maxerrors;
		return(pi);
		}
	return(NULL);
	}


void free_pi(struct prog_info *pi)
	{
	free(pi);
	}


void prepare_second_pass(struct prog_info *pi)
	{
	struct def *def, *temp_def;
	struct label *labelPtr, *temp_label;

	pi->segment = SEGMENT_CODE;
	pi->cseg_addr = 0;
	pi->dseg_addr = DSEG_START;
	pi->eseg_addr = 0;

	for(def = pi->first_def; def;)
		{
		temp_def = def;
		def = def->next;
		free(temp_def->name);
		free(temp_def);
		}
	pi->first_def = NULL;	
	pi->last_def = NULL;	

	for(labelPtr = pi->first_constant; labelPtr;)
		{
		temp_label = labelPtr;
		labelPtr = labelPtr->next;
		free(temp_label->name);
		free(temp_label);
		}
	pi->first_constant = NULL;
	pi->last_constant = NULL;

	for(labelPtr = pi->first_variable; labelPtr;)
		{
		temp_label = labelPtr;
		labelPtr = labelPtr->next;
		free(temp_label->name);
		free(temp_label);
		}
	pi->first_variable = NULL;
	pi->last_variable = NULL;
	}


void print_msg(struct prog_info *pi, int type, char *fmt, ... )
	{
	va_list argsPtr;
	char	errorString[200];
	char* 	fmtPtr;
	short 	argsIndex = 0;
	
	argsPtr = new char[200];

	if(type == MSGTYPE_OUT_OF_MEM)
				doCautionAlert("\pError: Unable to allocate memory!");
	else
		{
		unsigned char numString[10];
		va_start(argsPtr, fmt);
//		fprintf(stderr, "%s: %d: ", pi->fi->include_filePtr->name, pi->fi->line_number);
//		strcpy(errorString, pi->fi->include_filePtr->name);
		strcpy(errorString, "Error: ");
		strcat(errorString, pi->fi->include_filePtr->name);
		strcat(errorString, ":\rline ");
		NumToString((SInt32) pi->fi->line_number,(unsigned char*)numString);
		P2CStrCat(errorString, numString);
		strcat(errorString, ", ");
//		if(pi->macro_callPtr)
//			fprintf(stderr, "[Macro: %s: %d:] ", pi->macro_callPtr->macroPtr->include_filePtr->name, pi->macro_callPtr->line_index + pi->macro_callPtr->macroPtr->first_line_number);
//		strcat(errorString, pi->macro_callPtr->macroPtr->include_filePtr->name);
		strcat(errorString, fmt);
		for(argsIndex = 1; argsIndex < (*argsPtr - 1); argsIndex++)
		{
			fmtPtr = va_arg(argsPtr, char*);
			strcat(errorString, fmtPtr);
		}
		c2pstr(errorString);
//		doCautionAlert(c2pstr(strcat("Macro..",pi->macro_callPtr->macroPtr->include_filePtr->name)));
		doCautionAlert((unsigned char*)errorString);
		switch(type)
			{
			case MSGTYPE_ERROR:
				pi->error_count++;
//				fprintf(stderr, "Error: ");
				break;
			case MSGTYPE_WARNING:
				pi->warning_count++;
//				fprintf(stderr, "Warning: ");
				break;
			case MSGTYPE_MESSAGE:
				break;
			}
//		vfprintf(stderr, fmt, argsPtr);
//		strcpy(errorString, fmt);
//		c2pstr(errorString);
//		doCautionAlert((unsigned char*)errorString);
//		fprintf(stderr, "\n");
		va_end(argsPtr);
		}
	}




