/***********************************************************************
 *  avra - Assembler for the Atmel AVR microcontroller series
 *  Copyright (C) 1998-2000 Jon Anders Haugum
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to
 *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *
 *
 *  Author of avra can be reached at:
 *     email: jonah@omegav.ntnu.no
 *     www: http://www.omegav.ntnu.no/~jonah/el/avra.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "MacAVRpa.h"
#include "misc.h"
#include "args.h"
#include "avra.h"

int read_macro(struct prog_info *pi, char *name, int pass)
	{
	int loopok, i;
	struct macro *macroPtr;
	struct macro_line *macro_linePtr, **last_macro_line;

	if(pass == PASS_1)
		{
		if(!name)
			{
			print_msg(pi, MSGTYPE_ERROR, "Missing macro name");
			return(True);
			}
		get_next_token(name, TERM_END);
		// TODO: Sjekk om navnet er gyldig. Bare isalnum() og '_'
		macroPtr = (macro *)calloc(1, sizeof(struct macro));
		if(!macroPtr)
			{
			print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
			return(False);
			}
		if(pi->last_macro)
			pi->last_macro->next = macroPtr;
		else
			pi->first_macro = macroPtr;
		pi->last_macro = macroPtr;
		macroPtr->name = new char[strlen(name) + 1];
		if(!macroPtr->name)
			{
			print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
			return(False);
			}
		strcpy(macroPtr->name, name);
		macroPtr->include_filePtr = pi->fi->include_filePtr;
		macroPtr->first_line_number = pi->fi->line_number;
		last_macro_line = &macroPtr->first_macro_line;
		}
	else if(pi->list_line && pi->list_on) /* pass == PASS_2 */
		{
		fprintf(pi->list_file, "          %s", pi->list_line);
		pi->list_line = NULL;
		}
	loopok = True;
	while(loopok)
		{
		if(fgets(pi->fi->buff, LINEBUFFER_LENGTH, pi->fi->fp))
			{
			pi->fi->line_number++;
			i = 0;
			while(IS_HOR_SPACE(pi->fi->buff[i]) && !IS_END(pi->fi->buff[i])) i++;
			if(pi->fi->buff[i] == '.')
				{
				i++;
				if(!nocase_strncmp(&pi->fi->buff[i], "endm", 4)) // TODO: Vurder  sjekke navnet bedre
					loopok = False;
				}
			if(pass == PASS_1)
				{
				if(loopok)
					{
					macro_linePtr = (macro_line*)calloc(1, sizeof(struct macro_line));
					if(!macro_linePtr)
						{
						print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
						return(False);
						}
					*last_macro_line = macro_linePtr;
					last_macro_line = &macro_linePtr->next;
//					macro_linePtr->line = malloc(strlen(pi->fi->buff) + 1);
					macro_linePtr->line = new char[strlen(pi->fi->buff) + 1];
					if(!macro_linePtr->line)
						{
						print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
						return(False);
						}
					strcpy(macro_linePtr->line, pi->fi->buff);
					}
				}
			else if(pi->fi->buff && pi->list_file)
				{
				if(pi->fi->buff[i] == ';')
					fprintf(pi->list_file, "         %s", pi->fi->buff);
				else
					fprintf(pi->list_file, "          %s", pi->fi->buff);
				}
			}
		else
			{
			if(feof(pi->fi->fp))
				{
				print_msg(pi, MSGTYPE_ERROR, "Found no closing .ENDMACRO");
				return(True);
				}
			else
				{
//				perror(pi->fi->include_filePtr->name);
				char errorString[200];
				strcpy(errorString, pi->fi->include_filePtr->name);
				C2PStr(errorString);
				doCautionAlert((unsigned char*) errorString);
				return(False);
				}
			}
		}
	return(True);
	}


struct macro *get_macro(struct prog_info *pi, char *name)
	{
	struct macro *macro;

	for(macro = pi->first_macro; macro; macro = macro->next)	
		if(!nocase_strcmp(macro->name, name))
			return(macro);
	return(NULL);
	}


int expand_macro(struct prog_info *pi, struct macro *macroPtr, char *rest_line, int pass)
	{
	int ok = True, macro_arg_count = 0, i, j;
	char *line, *temp, *macro_args[10];
	char buff[LINEBUFFER_LENGTH];
	struct macro_line *old_macro_line;
	struct macro_call *macro_callPtr;

	if(rest_line)
		{
//		line = malloc(strlen(rest_line) + 1);
		line = new char[strlen(rest_line) + 1];
		if(!line)
			{
			print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
			return(False);
			}
		strcpy(line, rest_line);
		temp = line;
		while(temp)
			{
			macro_args[macro_arg_count++] = temp;
			temp = get_next_token(temp, TERM_COMMA);
			}
		}

	if(pass == PASS_1)
		{
		macro_callPtr = (macro_call *)calloc(1, sizeof(struct macro_call));
		if(!macro_callPtr)
			{
			print_msg(pi, MSGTYPE_OUT_OF_MEM, NULL);
			return(False);
			}
		if(pi->last_macro_call)
			pi->last_macro_call->next = macro_callPtr;
		else
			pi->first_macro_call = macro_callPtr;
		pi->last_macro_call = macro_callPtr;
		macro_callPtr->line_number = pi->fi->line_number;
		macro_callPtr->include_filePtr = pi->fi->include_filePtr;
		macro_callPtr->macroPtr = macroPtr;
		macro_callPtr->prev_on_stack = pi->macro_callPtr;
		if(macro_callPtr->prev_on_stack)
			{
			macro_callPtr->nest_level = macro_callPtr->prev_on_stack->nest_level + 1;
			macro_callPtr->prev_line_index = macro_callPtr->prev_on_stack->line_index;
			}
		}
	else
		{
		for(macro_callPtr = pi->first_macro_call; macro_callPtr; macro_callPtr = macro_callPtr->next)
			{
			if((macro_callPtr->include_filePtr->num == pi->fi->include_filePtr->num)
			   && (macro_callPtr->line_number == pi->fi->line_number))
				{
				if(pi->macro_callPtr)
					{
					/* Find correct macro_call when using recursion and nesting */
					if(macro_callPtr->prev_on_stack == pi->macro_callPtr)
						if((macro_callPtr->nest_level == (pi->macro_callPtr->nest_level + 1))
						   && (macro_callPtr->prev_line_index == pi->macro_callPtr->line_index))
							break;
					}
				else
					break;
				}
			}
		if(pi->list_line && pi->list_on)
			{
			fprintf(pi->list_file, "%06x   +  %s", pi->cseg_addr, pi->list_line);
			pi->list_line = NULL;
			}
		}
	macro_callPtr->line_index = 0;
	pi->macro_callPtr = macro_callPtr;
	old_macro_line = pi->macro_linePtr;
	for(pi->macro_linePtr = macroPtr->first_macro_line; pi->macro_linePtr && ok; pi->macro_linePtr = pi->macro_linePtr->next)
		{
		macro_callPtr->line_index++;
		if(GET_ARG(pi->argsPtr, ARG_LISTMAC))
			pi->list_line = buff;
		else
			pi->list_line = NULL;
		for(i = 0, j = 0; pi->macro_linePtr->line[i] != '\0'; i++)
			{
			if(pi->macro_linePtr->line[i] == '@')
				{
				i++;
				if(!isdigit(pi->macro_linePtr->line[i]))
					print_msg(pi, MSGTYPE_ERROR, "@ must be followed by a number");
				else if((pi->macro_linePtr->line[i] - '0') >= macro_arg_count)
					print_msg(pi, MSGTYPE_ERROR, "Missing macro argument (for @%c)", pi->macro_linePtr->line[i]);
				else
					{
					strcpy(&buff[j], macro_args[pi->macro_linePtr->line[i] - '0']);
					j += strlen(macro_args[pi->macro_linePtr->line[i] - '0']);
					}
				}
			else
				buff[j++] = pi->macro_linePtr->line[i];
			}
		buff[j] = '\0';
		ok = parse_line(pi, buff, pass);
		if(ok)
			{
			if((pass == PASS_2) && pi->list_line && pi->list_on)
				fprintf(pi->list_file, "         %s", pi->list_line);
			if(pi->error_count >= pi->max_errors)
				{
				print_msg(pi, MSGTYPE_MESSAGE, "Maximum error count reached. Exiting...");
				break;
				}
			}
		}
	pi->macro_linePtr = old_macro_line;
	pi->macro_callPtr = macro_callPtr->prev_on_stack;
	if(rest_line)
	        free(line);
	return(ok);
	}
