/***********************************************************************
 *  args - Argument reading system
 *  Copyright (C) 1997-1998 Jon Anders Haugum
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to
 *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *
 *
 *  Author of args can be reached at:
 *     email: jonah@colargol.tihlde.hist.no
 *     www: http://www.colargol.tihlde.hist.no/~jonah/el/avrp.html
 *     Postal address: Jon Anders Haugum
 *                     vre Mllenbergsgt 52
 *                     7014 Trondheim
 */
 // count

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <types.h>
#include <MacMemory.h>

#include "misc.h"
#include "avrpargs.h"


void * GET_avrparg(avrpargs* theargs,int argnum)
{
	return (theargs->thearg[argnum].data);
};

void SET_avrparg(avrpargs * theargs,int argnum,void * value) 
{
	theargs->thearg[argnum].data = (void *)value;
};


struct avrpargs *alloc_avrpargs(int arg_count)
	{
	struct avrpargs *theargs;

	theargs = new avrpargs;
	if(theargs)
		{
		
		theargs->thearg = new avrparg[arg_count];
		if(theargs->thearg)
			{
			theargs->count = arg_count;
			theargs->first_data = NULL;
			return(theargs);
			}
		delete theargs;
		}
	printf("Error: Unable to allocate memory\n");
	return(NULL);
	}




int read_avrpargs(struct avrpargs *theargs, int argc, char *argv[])
	{
	int i, j, k, ok = True, i_old;
	struct data_list **last_data, *data;
	char * thePtr;



	theargs->first_data = NULL;
	last_data = &theargs->first_data;
	for(i = 1; (i < argc) && ok; i++)
		{
		if(argv[i][0] == '-')
			{
			if(argv[i][1] == '-')
				{
				j = 0;
				while((theargs->thearg[j].longarg ? strcmp(&argv[i][2], theargs->thearg[j].longarg) : True) && (j != theargs->count))
					j++;
				if(j == theargs->count)
					{
					printf("Error: Unknown option: %s\n", argv[i]);
					ok = False;
					}
				else
					{
					switch(theargs->thearg[j].type)
						{
						case ARGTYPE_STRING:
							if((i + 1) == argc)
								{
								printf("Error: No argument supplied with option: %s\n", argv[i]);
								ok = False;
								}
							else
								theargs->thearg[j].data = argv[++i];
							break;
						case ARGTYPE_BOOLEAN:
							thePtr = new char;
							*thePtr = 1;
							theargs->thearg[j].data = thePtr;
							break;
						case ARGTYPE_STRING_MULTI:
							last_data = (struct data_list **)&theargs->thearg[j].data;
							break;
						}
					}
				}
			else
				{
				last_data = &theargs->first_data;
				// ToDo: Sjekk for enkel
				for(k = 1, i_old = i; (argv[i][k] != '\0') && ok && (i == i_old); k++)
					{
					j = 0;
					while((argv[i][k] != theargs->thearg[j].letter) && (j != theargs->count))
					{
						j++;
					}
					if(j == theargs->count)
						{
						printf("Error: Unknown option: -%c\n", argv[i][k]);
						ok = False;
						}
					else
						{
						switch(theargs->thearg[j].type)
							{
							case ARGTYPE_STRING:
								if(argv[i][k + 1] != '\0')
									{
									printf("Error: Option -%c must be followed by it's argument\n", argv[i][k]);
									ok = False;
									}
								else
									{
									if((i + 1) == argc)
										{
										printf("Error: No argument supplied with option: -%c\n", argv[i][k]);
										ok = False;
										}
									else
										theargs->thearg[j].data = argv[++i];
									}
								break;
							case ARGTYPE_BOOLEAN:
								theargs->thearg[j].data = (char *)True;
								break;
							case ARGTYPE_STRING_MULTI:
								last_data = (struct data_list **)&theargs->thearg[j].data;
								break;
							}
						}
					}
				}
			}
		else
			{
			while(*last_data) last_data = &((*last_data)->next);
			data = new data_list;
			if(data)
				{
				data->next = NULL;
				data->data = argv[i];
				*last_data = data;
				last_data = &data->next;
				}
			else
				{
				printf("Error: Unable to allocate memory\n");
				ok = False;
				}
			}
		}
	return(ok);
	}


void free_avrpargs(struct avrpargs *theargs)
	{
	int i;
	struct data_list *data, *temp;

	for(data = theargs->first_data; data;)
		{
		temp = data;
		data = data->next;
		delete temp;
		}
	for(i = 0; i != theargs->count; i++)
		if(theargs->thearg[i].type == ARGTYPE_STRING_MULTI)
			for(data = (struct data_list *)theargs->thearg[i].data; data;)
				{
				temp = data;
				data = data->next;
				delete(temp);
				}
	delete(theargs);
	}


void define_avrparg(struct avrpargs *theargs, int index, int type, char letter, char *longarg, void *def_value)
	{
	theargs->thearg[index].type = type;
	theargs->thearg[index].letter = letter;
	theargs->thearg[index].longarg = longarg;
	theargs->thearg[index].data = def_value;
	}
