/***********************************************************************
 *  MacAVRpa - Assembler and programmer for the Atmel AVR microcontroller 
 *  series
 *  Copyright (C) 1999 Ben Hitchcock
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to
 *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *
 *  Author of MacAVRpa can be reached at:
 *     email: ben@wollongong.apana.org.au
 *     Postal address: Ben Hitchcock
 *     C/0  Don Hitchcock
 *          "Fangorn", Old Inverell Rd
 *          Armidale
 *          NSW 2350
 *          AUSTRALIA
*/

#include <stdio.h>
#include <stdlib.h>

#include <serial.h>
#include <string.h>
#include <memory.h>
#include <CrmSerialDevices.h>
#include <CommResources.h>
#include <DriverServices.h>
#include "macAVRpa.h"
#include "io.macos.h"
#include "avrp.h"
#include "Miscellany.h"


struct Ser
	{
   char *Name;           
   long * SerRefNum;     
   int comm_open;        
   };
   
union dualdata
	{
	long lg; 
	struct {short inrefnum, outrefnum;} sg; 
	};
		
/************************************************************
		AVRP specific routines
		
		Ben Hitchcock   12/6/99
************************************************************/

void *OpenSer(char *SerPort)
   {
   struct Ser *theSer = NULL;
   OSErr theError;
   long* SerRefNumPtr;
   bool		foundPort = false;
   char portName[50];
   unsigned char pSerPort[50];
   
   unsigned char inputDriverName[100];
   unsigned char outputDriverName[100];
   
   theSer = new Ser;
   theSer->comm_open = 0;
   
   theSer->SerRefNum = new long;
   
	CRMRec 							crmRec;
	CRMRecPtr						crm=&crmRec;
	
	crmRec.crmDeviceType=crmSerialDevice;
	crmRec.crmDeviceID=0;
	
	strcpy(( char*)pSerPort, SerPort);
	c2pstr((char*)pSerPort);
	
	while ((!foundPort) && (crm=CRMSearch(crm)))
	{
		CRMSerialPtr portInfo =(CRMSerialPtr)crm->crmAttributes;
		
//		if(PStrCmp(pSerPort, *(portInfo->name)) == 0)

		PStrCopy((unsigned char*)portName, *(portInfo->name));
		P2CStr((unsigned char*)portName);

//		portName = p2cstr(*portInfo->name);

		if(strcmp(SerPort, portName) == 0)
		{
			foundPort = true;
			PStrCopy(inputDriverName, *(portInfo->inputDriverName));
			PStrCopy(outputDriverName, *(portInfo->outputDriverName));
		}
	}
	if(foundPort)
	{
		theError = serialOpen(((struct Ser *)theSer)->SerRefNum,inputDriverName, outputDriverName ,gThePrefs.portSpeed);	

	
		if(theError != 0)
		{
			SerRefNumPtr = ((struct Ser *)theSer)->SerRefNum;
			serialClose(*SerRefNumPtr); 
			SysBeep(1);
			doCautionAlert("\pError opening serial port.");
			delete theSer;
			theSer = NULL;
	/*		ExitToShell();	*/				
		}
		theSer->comm_open = 1;
	
//	printf("Opening serial port now...\n");
	} else
	{
		doCautionAlert("\pSerial Port not found.  Check the port in the preferences.");
		delete theSer;
		theSer = NULL;
	}
   return(theSer);
   }
		
void CloseSer(void *theSer)
   {
   long* SerRefNumPtr;
   
   SerRefNumPtr = ((struct Ser *)theSer)->SerRefNum;
   if(((struct Ser *)theSer)->comm_open)
      {
      serialClose(*SerRefNumPtr);
      }
//	printf("Closing serial port now...\n");
   delete (Ser*)theSer;
   }

void SerWriteByte(void * theSer, unsigned char Byte)
   {
   OSErr theError;
   long* SerRefNumPtr;
   struct Ser * mySer = (Ser *)theSer;
   
   SerRefNumPtr = mySer->SerRefNum;   
   
   theError = serialSendChar(*SerRefNumPtr, Byte);

	if(theError != 0)
		{
		SerRefNumPtr = mySer->SerRefNum;
		serialClose(*SerRefNumPtr); 
		SysBeep(1);
		doCautionAlert("\pError writing to serial port.");
//        printf("Error %o writing to serial port\n", theError);
/*		ExitToShell();*/					
		}
	
   }
		
	
int SerReadData(void *theSer, unsigned char *Buff, int Count)
   {
   int NumberOfChars = 0;
   OSErr theError;
   long* SerRefNumPtr;
   char outputBuffer[256];
   long NumberOfTrys = 0;
   unsigned char* theBuff;
   struct Ser * mySer = (Ser *)theSer;
   long theTickCount;
   
   SerRefNumPtr = mySer->SerRefNum;

	theBuff = Buff;
	*theBuff = 'a';
	while((NumberOfChars < Count))
	{
		
		theError = serialReadChar(*SerRefNumPtr, theBuff);
		if((*theBuff != '\0') || (NumberOfTrys > 60))
		{
			outputBuffer[NumberOfChars] = *theBuff;
			theBuff++;
			NumberOfChars++;	
		} else
		{
			theTickCount = TickCount();
			while(theTickCount == TickCount())
			{
			}
//			serialTimeout(*SerRefNumPtr,0, 2);
		};
		
		if(theError != 0)
		{
			SerRefNumPtr = mySer->SerRefNum;
			serialClose(*SerRefNumPtr); 
			SysBeep(1);
			doCautionAlert("\pError reading from serial port.");
//	        printf("Error reading from serial port\n");
	        
/*			ExitToShell();	*/				
		}
		NumberOfTrys++;
	}
	*theBuff = '\0';
	theBuff++;
	
	outputBuffer[NumberOfChars] = '\0';
//	printf(", Chars read in = %o, Count = %o", NumberOfChars, Count);
//	printf(", Read in: ");
//	printf(outputBuffer);
   return(NumberOfChars);
   }
	
/* Rudimentary serial port drivers for Metrowerks C

// Copyright  1997 by Francis J. Deck.
// It is OK to use this code, but only "at your own risk."

// This is a modest but fairly rigorous serial port driver. I wrote it to
// use in some software that I publish as Freeware with all of the usual
// disclaimers. An unusual feature of this driver is that it uses a single
// long-integer reference number for the ports, which your program must
// store and maintain. The reason for this is to allow incorporation of
// this code into code resources, which are not allowed to have global
// variables.

// All routines have error checking. Only the routine serialOpen() actually
// closes the ports if it encounters an error. Your program _must_ make sure
// that the ports are properly closed before terminating, or the open
// serial handlers will simply lurk on your Mac until you restart.

// We use only the default 64-byte buffers. That's OK for my purposes,
// since my data aquisition boards typically process commands with no more
// than half a dozen characters.

// There is no error checking for valid parameters. Also, the only routine
// that closes the ports upon encountering an error is serialOpen(). In
// general, your program must close the ports before terminating, or risk
// leaving an open port hanging on the system until restart.



// Close the serial port
*/
void serialClose
	(long refnum)
	
	{
	union dualdata dd; 
	
	dd.lg = refnum;
	CloseDriver (dd.sg.inrefnum); 
	CloseDriver (dd.sg.outrefnum);
	}
	

OSErr serialOpen
	(long *refnum, 
	unsigned char* DriverInName, 
	unsigned char* DriverOutName, 
	long baud) 
	
	{
	
	union dualdata dd;
	
//	dd = new dualdata;
	short baudval; 
	SerShk ss; 
		
	if (OpenDriver(DriverOutName, &dd.sg.outrefnum) != noErr) return yesErr;
			
	if (OpenDriver(DriverInName, &dd.sg.inrefnum) != noErr) 
		{
		CloseDriver (dd.sg.outrefnum); 
		return yesErr;
		}
	
	ss.fXOn = 0; 
	ss.xOn = 19; 
	ss.xOff = 17; 
	ss.fCTS = 0; 
	ss.errs = 0; 
	ss.evts = 0; 
	ss.fDTR = 0; 
		
	if (SerHShake(dd.sg.inrefnum, &ss) != noErr)
		{
		serialClose(dd.lg);
		return yesErr;
		}
	
	if (SerHShake(dd.sg.outrefnum, &ss) != noErr)
		{
		serialClose(dd.lg);
		return yesErr;
		}

		
	switch (baud)
		{
		case 300: { baudval = baud300; break; }
		case 600: { baudval = baud600; break; }
		case 1200: { baudval = baud1200; break; }
		case 1800: { baudval = baud1800; break; }
		case 2400: { baudval = baud2400; break; }
		case 3600: { baudval = baud3600; break; }
		case 4800: { baudval = baud4800; break; }
		case 7200: { baudval = baud7200; break; }
		case 9600: { baudval = baud9600; break; }
		case 14400: { baudval = 6; break; }
		case 19200: { baudval = baud19200; break; }
		case 28800: { baudval = 2; break; }
		case 38400: { baudval = baud38400; break; }
		case 57600: { baudval = baud57600; break; }		
		}
	

	
	if (SerReset(dd.sg.inrefnum, baudval + stop10 + noParity + data8) != noErr)
		{
		serialClose(dd.lg);
		return yesErr;
		}
		
	if (SerReset(dd.sg.outrefnum, baudval + stop10 + noParity + data8) != noErr)
		{
		serialClose(dd.lg);
		return yesErr;
		}
		
	*refnum = dd.lg;
	
	return noErr;
	}
	

OSErr serialNAvail
	(
	long refnum, 
	long *navail 
	)
	
	{
	union dualdata dd; 
	
	dd.lg = refnum;
	
	if (SerGetBuf(dd.sg.inrefnum, navail) != noErr)
		{
		serialClose(refnum);
		return yesErr;
		}
	return noErr;
	}
	

OSErr serialReadChar
	(long refnum,
	unsigned char *c) 
	{
	union dualdata dd; 
	long cn;
	
	dd.lg = refnum;
	

	
	if (SerGetBuf(dd.sg.inrefnum, &cn) != noErr) return yesErr;
	if (cn < 1)
		{
		*c = 0;
		return noErr;
		}
	

	
	cn = 1; 	
	if (FSRead(dd.sg.inrefnum, &cn, c) != noErr) return yesErr;	
	return noErr;
	}
	


OSErr serialSendChar
	(long refnum, 
	unsigned char c) 
	
	{
	union dualdata dd; 
	long cn;
	OSErr theErr;
	
				doIndicatorIterate();
	dd.lg = refnum;
	cn = 1;
	theErr = FSWrite(dd.sg.outrefnum, &cn, &c);
	if(theErr != noErr) return yesErr;
	return noErr;
	}
	


OSErr serialFlush
	(long refnum) 
	
	{
	union dualdata dd;
	long cn = 0;
	char c; 
	
	dd.lg = refnum;
	

	
	if (SerGetBuf(dd.sg.inrefnum, &cn) != noErr) return yesErr;
	

	
	while (cn > 0)
		{
		cn = 1; 
		

		
		if (FSRead(dd.sg.inrefnum, &cn, &c) != noErr) return yesErr;
		

		
		if (SerGetBuf(dd.sg.inrefnum, &cn) != noErr) return yesErr;
		}
	return noErr;
	}
	


OSErr serialTimeout
	(long refnum,
	long howmany, 
	long nticks)
	
	{
	long timeout; 
	long nchars = 0;
	
	timeout = TickCount() + nticks;
		
	while (nchars < howmany)
		{
		if (TickCount() > timeout) return yesErr;
		if (serialNAvail(refnum, &nchars) != noErr) return yesErr;
		}
		
	return noErr;
	}
	


OSErr serialXmitWait
	(long refnum)
	
	{
	union dualdata dd;
	SerStaRec ss;
	
	dd.lg = refnum;
	if (SerStatus (dd.sg.outrefnum, &ss) != noErr) return yesErr
	
	while (ss.wrPend != 0)
		if (SerStatus (dd.sg.outrefnum, &ss) != noErr) return yesErr;
		
	return noErr;
	}